dnl Copyright (C) 1997-2003 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
dnl  
dnl This file is free software; as a special exception the author gives
dnl unlimited permission to copy and/or distribute it, with or without 
dnl modifications, as long as this notice is preserved.
dnl 
dnl This program is distributed in the hope that it will be useful, but
dnl WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
dnl implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
dnl 

dnl
dnl readline support
dnl
dnl	defines READLINE_LIB, e.g. /usr/local/lib/libreadline.a
dnl		READLINE_INCLUDE, e.g.   -I/usr/local/include
dnl		READLINE_DIR, e.g. /usr/local
dnl		READLINE_SO, e.g. -L$(READLINE_DIR)/lib -lreadline
dnl
AC_DEFUN(AC_LIB_READLINE,
[
  READLINE_DIR=""
  READLINE_INCLUDE=""
  READLINE_LIB=""
  READLINE_SO=""

  AC_ARG_WITH(readline_dir,
    AC_HELP_STRING([--with-readline-dir=DIR],
      [Directory prefix where include/readline/readline.h is installed]),
    readlinesearch="$withval", 
    readlinesearch=""
  )

  AC_MSG_CHECKING([for readline])
  for dir in $readlinesearch $prefix /usr /opt /usr/local; do
    if test -r $dir/include/readline/readline.h; then
      READLINE_DIR=$dir
      break
    fi
  done
  
  if test -z "$READLINE_DIR"; then
    AC_MSG_ERROR([
      readline not found
    ])
  fi

  AC_MSG_RESULT([$READLINE_DIR/include/readline])
  if test "$READLINE_DIR" = "/usr" -o "$READLINE_DIR" = "/opt"; then
    dnl AC_MSG_NOTICE([no include path for readline needed])
    READLINE_INCLUDE=""
    READLINE_SO="-lreadline"
  else
    READLINE_INCLUDE="-I$READLINE_DIR/include"
    READLINE_SO="-R $READLINE_DIR/lib -lreadline"
  fi
  READLINE_LIB="$READLINE_DIR/lib/libreadline.a"

  AC_SUBST(READLINE_DIR)
  AC_SUBST(READLINE_LIB)
  AC_SUBST(READLINE_INCLUDE)
  AC_SUBST(READLINE_SO)
])

dnl
dnl tcl support
dnl
dnl	defines TCL_INCLUDE, e.g. -I/usr/include/tcl8.3
dnl		TCL_DIR, e.g. /usr/lib/tcl8.3
dnl
AC_DEFUN(AC_LIB_TCL,
[
  AC_ARG_WITH(tcldir,
    AC_HELP_STRING([--with-tcldir=DIR],
      [directory containing tclConfig.sh]),
    tcldirsearch="$withval",
    tcldirsearch=""
  )
  AC_MSG_CHECKING([for tcl])
  tmp=`ls -d /usr/lib/tcl?.?|sort -r`
  TCL_DIR=""
  for dir in $tcldirsearch $prefix /usr/lib $tmp /opt/lib \
	     /opt/lib/tcl* /usr/local/lib /usr/local/lib/tcl* 
  do
    file=$dir/tclConfig.sh
    if test -f $file; then
      TCL_DIR="$dir"
      break
    fi
  done

  if test -z "$TCL_DIR"; then
    AC_MSG_RESULT([no])
    AC_MSG_ERROR([
      tcl no found
    ])
  fi
  AC_MSG_RESULT([$TCL_DIR])
  . $TCL_DIR/tclConfig.sh
  

  TCL_INCLUDE=$TCL_INCLUDE_SPEC
  
  if test -z "$TCL_INCLUDE"; then
     TCL_INCLUDE=`echo -I$TCL_DIR | sed s/lib/include/`
  fi

  AC_SUBST(TCL_DIR)
  AC_SUBST(TCL_INCLUDE)
])

dnl
dnl check for a terminal library.
dnl
dnl	defines TERMINAL_LIB, e.g. -lncurses
dnl
AC_DEFUN(AC_LIB_TERMINAL,
[
  TERMINAL_LIB=""

  for tlib in ncurses termlib termcap curses; do
    AC_CHECK_LIB($tlib, tputs, [
      TERMINAL_LIB="-l$tlib"
      break
    ])
  done

  if test -z "$TERMINAL_LIB"; then
    AC_MSG_ERROR([
      no terminal library found.
    ])
  fi

  AC_SUBST(TERMINAL_LIB)
])

dnl
dnl check for dynamic linkage library
dnl
dnl	defines DYN_LIB, e.g. /sw/lib/libdl.dylib
dnl
AC_DEFUN(AC_LIB_DYN,
[
  DYN_LIB=""

  AC_ARG_WITH(dynlib_dir,
    AC_HELP_STRING([--with-dynlib-dir=DIR],
      [Directory prefix where libdl.so is installed]),
    dynlibsearch="$withval", 
    dynlibsearch=""
  )


  AC_CHECK_LIB(c, dlopen,
    DYN_LIB="",
    AC_CHECK_LIB(dl, dlopen,
      DYN_LIB="-ldl", [
	AC_MSG_CHECKING([libdl.a])
	dynlibdir=""
	for dir in $dynlibsearch; do
	  if test -r $dir/libdl.a; then
	    dynlibdir="$dir"
	    DYN_LIB="-L$dir -ldl"
	    break
	  fi
	done

	if test -z "$DYN_LIB"; then
	  AC_MSG_RESULT([no])
	  AC_MSG_ERROR([
	    Dynamic linker needed. Try --with-dynlib-dir.
	  ])
	fi
	AC_MSG_RESULT([$dynlibdir])
      ]
    )
  )

  AC_SUBST(DYN_LIB)
])

dnl
dnl check for perl and its core
dnl
dnl	defines PERL, e.g. /usr/bin/perl
dnl	        PERL_CORE, e.g. /usr/lib/perl5/5.6.1/i586-linux/CORE
dnl		PERL_INCLUDE, -I$(PERL_CORE)
dnl		PERL_LIBS, e.g. -ldb -ldl -lm -lpthread -lc -lcrypt
dnl		PERL_FLAGS, e.g. -D_REENTRANT -D_GNU_SOURCE -DDEBIAN 
dnl                              -fno-strict-aliasing -D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64
dnl
AC_DEFUN(AC_PERL,
[
  PERL=""
  PERL_CORE=""
  PERL_INCLUDE=""
  PERL_LIBS=

  AC_ARG_WITH(perl_dir,
    AC_HELP_STRING([--with-perl-dir=DIR],
      [Directory prefix where bin/perl is installed]),
    perlsearch="$withval", 
    perlsearch=""
  )

  if test -z "$perlsearch"; then
    searchpath=$PATH
  else
    searchpath=$perlsearch/bin:$PATH
  fi

  AC_PATH_PROG(PERL,perl,,$searchpath)

  if test -z "$PERL"; then
    AC_MSG_ERROR([
      perl not found
    ])
  fi

  AC_MSG_CHECKING([for perl CORE])
  PERL_CORE=`$PERL -MConfig -e 'print "$Config{archlibexp}/CORE"'`
  PERL_FLAGS=`$PERL -MConfig -e 'print "$Config{ccflags}"'`
  PERL_LIBS=`$PERL -MConfig -e 'print "$Config{perllibs}"'`
  PERL_INCLUDE="-I$PERL_CORE"

  if test -z "$PERL_CORE"; then
    AC_MSG_RESULT([no])
    AC_MSG_ERROR([
      perl CORE not found
    ])
  fi
  AC_MSG_RESULT([$PERL_CORE])


  dnl tmp_CPPFLAGS=$CPPFLAGS
  dnl CPPFLAGS="$CPPFLAGS $PERL_FLAGS $PERL_INCLUDE"
  dnl AC_CHECK_HEADER(perl.h, [], [AC_MSG_ERROR(Could not find perl.h)])
  dnl AC_CHECK_HEADER(EXTERN.h, [], [AC_MSG_ERROR(Could not find EXTERN.h)])
  dnl AC_CHECK_HEADER(XSUB.h, [], [AC_MSG_ERROR(Could not find XSUB.h)])
  dnl CPPFLAGS=$tmp_CPPFLAGS

  AC_SUBST(PERL_CORE)
  AC_SUBST(PERL_INCLUDE)
  AC_SUBST(PERL_FLAGS)
  AC_SUBST(PERL_LIBS)
])

dnl
dnl check for the right version of swig
dnl
dnl	defines SWIG, e.g. /usr/local/bin/swig
dnl		OLD_SWIG (a conditional being true if we detected swig < 1.3)
AC_DEFUN(AC_SWIG,
[
  AC_CHECK_PROG(SWIG,swig,swig)

  if test -z $SWIG; then
    AC_MSG_RESULT([no])
    AC_MSG_ERROR([
      SWIG not found
    ])
  fi

  AC_MSG_CHECKING([for swig])
  SWIG_VERSION="`$SWIG -version 2>&1 | grep SWIG.Version | sed 's/^SWIG Version \([[0-9]]\.[[0-9]]\).*$/\1/'`"
  AC_MSG_RESULT([found version $SWIG_VERSION])

  AM_CONDITIONAL(OLD_SWIG, test x"$SWIG_VERSION" = "x1.1")
  AH_TEMPLATE(OLD_SWIG, [defined if we still use swig version < 1.3])
  if test "x$OLD_SWIG_TRUE" = "x"; then
    AC_DEFINE_UNQUOTED(OLD_SWIG)
  fi
])

dnl
dnl check for the berkley db
dnl
dnl	defines BDB_INCLUDE, e.g. -I/opt/pkg/db-3.2.9/include
dnl		BDB_LIB, e.g. -ldb
dnl		DBD_DIR, e.g. /usr
AC_DEFUN(AC_LIB_BDB,[
  BDB_INCLUDE=""
  BDB_LIB=""
  BDB_DIR=""

  AC_ARG_WITH([berkeley-db],
    AC_HELP_STRING([--with-berkeley-db=DIR],
      [use Berkeley DB located in DIR]),
    [bdbsearch="$withval"],
    [bdbsearch=""])

  AC_MSG_CHECKING([for Berkeley DB])
  for dir in $bdbsearch $prefix /usr /opt /usr/local; do
    if test -r $dir/lib/libdb.so; then
      BDB_DIR=$dir
      break
    fi
  done

  if test -z "$BDB_DIR"; then
    AC_MSG_RESULT([no])
    AC_MSG_ERROR([
      Berkley DB not found
    ])
  fi

  AC_MSG_RESULT([$BDB_DIR])
  if test "$BDB_DIR" = "/usr" -o "$BDB_DIR" = "/opt"; then
    dnl AC_MSG_NOTICE([no include path for berkeley db needed])
    BDB_INCLUDE=""
    BDB_LIB="-ldb"
  else
    BDB_INCLUDE="-I$BDB_DIR/include"
    BDB_LIB="-L$BDB_DIR/lib -ldb"
  fi


  AC_SUBST(BDB_DIR)
  AC_SUBST(BDB_LIB)
  AC_SUBST(BDB_INCLUDE)

])
