/* Copyright (C) 1997-2003 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without 
 * modifications, as long as this notice is preserved.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Michael Schulz (see also AUTHORS and THANKS for more)
 * Birth: 12 Juli 1998
 */

/* -------------------------------------------------------------------------
 * @addtogroup HookBindings
 * @ingroup Hook
 * @{
 */

#ifndef _HOOKER_H
#define _HOOKER_H
#include <stdarg.h>

#include <tcl.h>
#include <tk.h>

#undef None
#include <blah.h>
#include "hook.h"
#include "parse.h"


/* -- TYPE DEFINITIONS -------------------------------------------------- */

#ifndef SWIG

/* --------------------------------------------------------------------------
 * buffer of the output channel.
 */
typedef struct {
   int size;		/**< current size buffered */
   int maxsize;		/**< memory allocated in this buffer */
   char *buffer;	/**< a chunk of memory to buffer stuff in */
} LoggerStruct;
typedef LoggerStruct *Logger; /**< type of a Logger */

/* --------------------------------------------------------------------------
 * type of HookResults.
 */
typedef enum {
   HTNone,
   HTInt,
   HTString,
   HTError
} HookResultType;


/* --------------------------------------------------------------------------
 * typed result of a hook.
 * This structure tracks the result of a hook execution. The return value
 * stored within is used in our special typemaps for the swig interface 
 * generator.
 */
typedef struct {
  HookResultType type;		/**< type of the result data */
  /** union of possible results */
  union {			
    int intResult;		/**< int result value */
    char *stringResult;		/**< string result value */
  } data;
  Hook hook;			/**< the hook that produced this result */
} HookResultStruct;
typedef HookResultStruct *HookResult; /**< type of a HookResult */
#endif


/* --------------------------------------------------------------------------
 * tagging the tcl result type.
 * This typedef is used to allow special typemaps in the swig interface.
 */
typedef int TclResultType;


/* -- VARIABLES --------------------------------------------------------- */

#ifndef SWIG
extern Tcl_Interp *hkInterp;
extern HookResult hkResult;
#endif

/* -- FUNCTIONS ----------------------------------------------------------- */

int loggerSize(int);
TclResultType logWriteChar(char);
TclResultType setHookCmd(char *, char *);
TclResultType logWrite(char *);
TclResultType getHookCmd(char *);
TclResultType logPrintf(char *format, va_list ap);
TclResultType logFlush();

#ifndef SWIG
TclResultType glsInteractionHookHandle(Hook hook, va_list ap);
TclResultType evalHookHandle(Hook hook, va_list ap);
TclResultType netsearchHookHandle(Hook hook, va_list ap);
TclResultType getsHookHandle(Hook hook, String buffer, int size);
TclResultType progressHookHandle(Hook hook, char *format, va_list ap);
TclResultType partialResultHookHandle(Hook hook, va_list ap);
TclResultType ICinteractionHookHandle(Hook hook, va_list ap);
TclResultType tclHookHandle(Hook hook, va_list ap);
void hooker_init(Tcl_Interp * interp);
#endif


/* ---------------------------------------------------------------------- */
/** @} */
#endif /* don't insert anything after this #endif */
