/*
 * File: varsdemo.c
 * Author: Horia F. Pop
 * Purpose: demo for libcdg/vars.c
 * Birth: February 11, 2000
 * Changes:
 */

/* -- INCLUDES ---------------------------------------------------------- */
#include "configure.h"
#include <stdlib.h>
#include "variablesdemo.h"
#include "variables.h"
#include "cdg.h"
#include "hook.h"
#include <blah.h>

/* -- MACROS ------------------------------------------------------------ */

/* -- TYPE DEFINITIONS -------------------------------------------------- */

/* -- VARIABLES --------------------------------------------------------- */

/* -- FUNCTIONS --------------------------------------------------------- */

Boolean varFunction(int i)
{
  switch (i)
    {
    case 10:
    case 20:
    case 30:
      return TRUE;
    default:
      return FALSE;
    }
}

void varDemo()
{
  char sc1[] = "val-sc1";
  char sc2[] = "val-sc2";
  String s1 = NULL;
  String s2 = NULL;
  String s3 = NULL;
  int i1 = 10;
  int i2 = 20;
  List l = NULL;
  Boolean bi, bs;

  void varPrintVars(String s) {
    cdgPrintf(CDG_INFO, "%s\n", s);
    cdgPrintf(CDG_INFO, "s1 = `%s', s2 = `%s', s3=`%s'\n", s1, s2, s3);
    cdgPrintf(CDG_INFO, "i1 = %d, i2 = %d\n", i1, i2);
    cdgPrintf(CDG_INFO, "\n");
  }

  void varPrintBoolean(String s, Boolean b) {
    cdgPrintf(CDG_INFO, "%s ", s);
    if (b) {
      cdgPrintf(CDG_INFO, "success");
    } else {
      cdgPrintf(CDG_INFO, "failure");
    }
    cdgPrintf(CDG_INFO, "\n");
  }

  s1 = sc1;
  s2 = sc2;
  s3 = strRegister("val-s3-longer");

  varPrintVars("Initial");

  bi = varRegister("i1", VAR_INT, &i1, varFunction, NULL, NULL);
  bs = varRegister("s1", VAR_STRING, &s1, NULL, NULL, NULL);

  varPrintBoolean("varRegister on i1 returned", bi);
  varPrintBoolean("varRegister on s1 returned", bs);

  varPrintVars("After registering");

  bi = varSet("i1", 50);
  bs = varSet("s1", strRegister("long-string"));
  /* possible calls with string argument: varSet("s1", sc3); varSet("s1",
   * s3); varSet("s1", strRegister("long-string")); varSet("s1", "foo"); */

  varPrintBoolean("varSet on i1 returned", bi);
  varPrintBoolean("varSet on s1 returned", bs);

  varPrintVars("After set");

  bi = varGet("i1", &i2);
  bs = varGet("s1", &s2);

  varPrintBoolean("varGet on i1 returned", bi);
  varPrintBoolean("varGet on s1 returned", bs);

  varPrintVars("After get");

  s1 = strRegister("val-modif");
  i1 = 30;

  varPrintVars("After local change");

  cdgPrintf(CDG_INFO, "Contents of variables list\n");
  for (l = varReturnListOfNames(); l; l = listNext(l)) {
    String s = (String) listItem(l);

    switch (varReturnType(s)) {
      case VAR_INT:
	cdgPrintf(CDG_INFO, "%s = int\n", s);
	break;
      case VAR_FLOAT:
	cdgPrintf(CDG_INFO, "%s = float\n", s);
	break;
      case VAR_BOOL:
	cdgPrintf(CDG_INFO, "%s = Boolean\n", s);
	break;
      case VAR_STRING:
	cdgPrintf(CDG_INFO, "%s = String\n", s);
	break;
      default:
	cdgPrintf(CDG_INFO, "%s = ILLEGAL TYPE\n", s);
	break;
    }
  }

  cdgPrintf(CDG_INFO, "\nContents of variables list using varIterator object\n");
  {
    VarIterator vi = NULL;
    String s = NULL;

    for (varIteratorNew(&vi); (s = varIteratorNextName(&vi));) {
      cdgPrintf(CDG_INFO, " vi == %p\n", vi);
      cdgPrintf(CDG_INFO, "&vi == %p\n", &vi);
      switch (varReturnType(s)) {
	case VAR_INT:
	  cdgPrintf(CDG_INFO, "%s = int\n", s);
	  break;
	case VAR_FLOAT:
	  cdgPrintf(CDG_INFO, "%s = float\n", s);
	  break;
	case VAR_BOOL:
	  cdgPrintf(CDG_INFO, "%s = Boolean\n", s);
	  break;
	case VAR_STRING:
	  cdgPrintf(CDG_INFO, "%s = String\n", s);
	  break;
	default:
	  cdgPrintf(CDG_INFO, "%s = ILLEGAL TYPE\n", s);
	  break;
      }
    }
    cdgPrintf(CDG_INFO, " vi == %p\n", vi);
    cdgPrintf(CDG_INFO, "&vi == %p\n", &vi);
    varIteratorFree(&vi);
  }

}
