/////////////////////////////////////////////////////////////////////////////
// Makros
/////////////////////////////////////////////////////////////////////////////

m4_ifdef(_MAKROS,,`
m4_define(_MAKROS,1)

//////////////////////////////////////////////////////////////////////////////
// Hilfsmakros
//////////////////////////////////////////////////////////////////////////////
m4_define(`nshift', `m4_ifelse($1, 0, `m4_shift($@)', `nshift(m4_decr($1),m4_shift(m4_shift($@)))')')

//////////////////////////////////////////////////////////////////////////////
// Lexikon--Makros
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
// Kategorie NN
//////////////////////////////////////////////////////////////////////////////

// MakeNN (nur fuer normal nouns NN)
// Deklination schwach und neutral existiert nicht im Deutschen!
// Deklination gemischt und feminin existiert nicht im Deutschen!
// $1:	Wort im Nominativ singular
// $2:  Hier sollte in der Regel nur - stehen. Endet der Nominativ
//	singular des zu erzeugenden Wortes allerdings auf n und
//      hat dieses eine endungslose Pluralform (Bspl: Boden, Bden),
//      so sollte hier -n eingetragen werden. Dies stsst eine Son-
//      derbehandlung an, da die Deklinationsmakros ansonsten falsche
//     	Ausgaben erzeugen. Bei Bedarf knnen weitere Sonderbehandlung-
//  	en integriert werden.
// $3:	Wort im Nominativ plural
// $4:	Fast immer derjenige Suffix, welcher $1 und $3 unterscheidet,
//      eingeleitet durch - (Bspl: Baer, Baeren -> -en). Es gib aber
//	Ausnahmen (Bspl: Mensa, Mensen -> -en).
// $5:	Genus (masc o. fem o. neut)
// $6:	Deklinationsart (strong o. weak o. mixed)
// $7:  countable (yes o. no)
// $8:	timeunit (yes o. no)
// $9:	sort
// $10:	Anzahl Argumente (0 bis 2)
//
// Die folgenden Parameter (11 - 16) entfallen falls Anzahl Argumente = 0
// $11:	Penalty fuer das Fehlen des 1. Argumentes falls Wort im Singular
// $12:	Penalty fuer das Fehlen des 1. Argumentes falls Wort im Plural
// $13:	Kategorie (cat) des ersten Argumentes (Determiner o. Artikel)
//
// Die folgenden Parameter (14 - 16) entfallen falls Anzahl Argumente = 1
// $14:	Penalty fuer das Fehlen des 2. Argumentes falls Wort im Singular
// $15:	Penalty fuer das Fehlen des 2. Argumentes falls Wort im Plural
// $16:	Kategorie (cat) des zweiten Argumentes

m4_define(`MakeNN',
MakeOneNoun(declineNN_$6_$5($1, $2, $3, $4, nom, sg), $5, nom, $7, $8, $9, $10, $11, $13, $14, $16, sg)
MakeOneNoun(declineNN_$6_$5($1, $2, $3, $4, nom, pl), $5, nom, $7, $8, $9, $10, $12, $13, $15, $16, pl)
MakeOneNoun(declineNN_$6_$5($1, $2, $3, $4, gen, sg), $5, gen, $7, $8, $9, $10, $11, $13, $14, $16, sg)
MakeOneNoun(declineNN_$6_$5($1, $2, $3, $4, gen, pl), $5, gen, $7, $8, $9, $10, $12, $13, $15, $16, pl)
MakeOneNoun(declineNN_$6_$5($1, $2, $3, $4, dat, sg), $5, dat, $7, $8, $9, $10, $11, $13, $14, $16, sg)
MakeOneNoun(declineNN_$6_$5($1, $2, $3, $4, dat, pl), $5, dat, $7, $8, $9, $10, $12, $13, $15, $16, pl)
MakeOneNoun(declineNN_$6_$5($1, $2, $3, $4, acc, sg), $5, acc, $7, $8, $9, $10, $11, $13, $14, $16, sg)
MakeOneNoun(declineNN_$6_$5($1, $2, $3, $4, acc, pl), $5, acc, $7, $8, $9, $10, $12, $13, $15, $16, pl))


// MakeNNspecial (Verwenden falls `MakeNN' nicht anwendbar)
//
// $1:  Nominativ singular
// $2:  Genitiv singular
// $3:  Dativ singular
// $4:  Akkusativ singular
// $5:  Nominativ plural
// $6:  Genitiv plural
// $7:  Dativ plural
// $8:  Akkusativ plural
// $9:  Genus (masc o. fem o. neut)
// $10: countable (yes o. no)
// $11: timeunit (yes o. no)
// $12: sort
// $13: Anzahl der Argumente (0 bis 2)
//
// Die folgenden Parameter (14 - 19) entfallen falls Anzahl Argumente = 0
// $14:	Penalty fuer das Fehlen des 1. Argumentes falls Wort im Singular
// $15:	Penalty fuer das Fehlen des 1. Argumentes falls Wort im Plural
// $16:	Kategorie (cat) des ersten Argumentes (Determiner o. Artikel)
//
// Die folgenden Parameter (17 - 19) entfallen falls Anzahl Argumente = 1
// $17:	Penalty fuer das Fehlen des 2. Argumentes falls Wort im Singular
// $18:	Penalty fuer das Fehlen des 2. Argumentes falls Wort im Plural
// $19:	Kategorie (cat) des zweiten Argumentes


m4_define(`MakeNNspecial',
`m4_ifelse($1,`',,`MakeOneNoun($1, $9, nom, $10, $11, $12, $13, $14, $16, $17, $19, sg)')
m4_ifelse($2,`',,`MakeOneNoun($2, $9, gen, $10, $11, $12, $13, $14, $16, $17, $19, sg)')
m4_ifelse($3,`',,`MakeOneNoun($3, $9, dat, $10, $11, $12, $13, $14, $16, $17, $19, sg)')
m4_ifelse($4,`',,`MakeOneNoun($4, $9, acc, $10, $11, $12, $13, $14, $16, $17, $19, sg)')
m4_ifelse($5,`',,`MakeOneNoun($5, $9, nom, $10, $11, $12, $13, $15, $16, $18, $19, pl)')
m4_ifelse($6,`',,`MakeOneNoun($6, $9, gen, $10, $11, $12, $13, $15, $16, $18, $19, pl)')
m4_ifelse($7,`',,`MakeOneNoun($7, $9, dat, $10, $11, $12, $13, $15, $16, $18, $19, pl)')
m4_ifelse($8,`',,`MakeOneNoun($8, $9, acc, $10, $11, $12, $13, $15, $16, $18, $19, pl)')')


//MakeOneNoun (nur fuer normal nouns NN)
// $1:	Zu erzeugendes Wort
// $2:	Genus
// $3: 	Kasus
// $4: 	countable (yes o. no)
// $5:	timeunit (yes o. no)
// $6:	sort
// $7:  Anzahl Argumente
//
// Die folgenden Parameter sind evtl. leer
// $8:  Penalty fuer das Fehlen des 1. Argumentes
// $9:	Kategorie (cat) des ersten Argumentes (Determiner o. Artikel)
// $10: Penalty fuer das Fehlen des 2. Argumentes
// $11:	Kategorie (cat) des zweiten Argumentes
// $12: Numerus (sg o. pl)


m4_define(`MakeOneNoun',
`m4_ifelse(
   $7, 0,
`$1 :=
[ cat:NN,
  number:$12,
  case:$3,
  person:3,
  gender:$2,
  countable:$4,
  timeunit:$5,
  sort:$6
];',
   $7, 1,
`$1 :=
[ cat:NN,
  number:$12,
  case:$3,
  person:3,
  gender:$2,
  countable:$4,
  timeunit:$5,
  sort:$6,
  args:
  [ 1:
    [ realized:yes, obl:$8,
      cat:$9,
      case:$3,
      gender:$2,
      number:$12
    ]
  ]
];',
   $7, 2,
`$1 :=
[ cat:NN,
  number:$12,
  case:$3,
  person:3,
  gender:$2,
  countable:$4,
  timeunit:$5,
  sort:$6,
  args:
  [ 1:
    [ realized:yes, obl:$8,
      cat:$9,
      case:$3,
      gender:$2,
      number:$12
    ],
    2:
    [ realized:yes, obl:$10,
      cat:$11
    ]
  ]
];')')


//declineNN_strong_masc (nur fuer normal nouns NN)

m4_define(`declineNN_strong_masc',
`m4_ifelse(
  $6, pl,
   m4_ifelse(
      $4, -e,
      m4_ifelse($5, nom, $3,
		$5, gen, $3,
		$5, dat, $3n,
		$5, acc, $3),
      $4, -er,
      m4_ifelse($5, nom, $3,
		$5, gen, $3,
		$5, dat, $3n,
		$5, acc, $3),
      $4, -s,
      m4_ifelse($5, nom, $3,
		$5, gen, $3,
		$5, dat, $3,
		$5, acc, $3),
      $4, -,
      m4_ifelse(
	 $2, -n,
         m4_ifelse($5, nom, $3,
		   $5, gen, $3,
		   $5, dat, $3,
		   $5, acc, $3),
	 $2, -,
	 m4_ifelse($5, nom, $3,
		$5, gen, $3,
		$5, dat, $3n,
		$5, acc, $3))),
   $6, sg,
   m4_ifelse(
      $4, -e,
      m4_ifelse($5, nom, $1,
		$5, gen, $1es,
		$5, dat, $1,
		$5, acc, $1),
      $4, -er,
      m4_ifelse($5, nom, $1,
		$5, gen, $1es,
		$5, dat, $1,
		$5, acc, $1),
      $4, -s,
      m4_ifelse($5, nom, $1,
		$5, gen, $1s,
		$5, dat, $1,
		$5, acc, $1),
      $4, -,
      m4_ifelse($5, nom, $1,
		$5, gen, $1s,
		$5, dat, $1,
		$5, acc, $1)))')


//declineNN_strong_neut (nur fuer normal nouns NN)

m4_define(`declineNN_strong_neut',
`m4_ifelse(
   $6, sg,
   m4_ifelse(
      $4, -e,
      m4_ifelse($5, nom, $1,
		$5, gen, $1es,
		$5, dat, $1,
		$5, acc, $1),
      $4, -se,
      m4_ifelse($5, nom, $1,
		$5, gen, $1ses,
		$5, dat, $1,
		$5, acc, $1),
      $4, -er,
      m4_ifelse($5, nom, $1,
		$5, gen, $1es,
		$5, dat, $1,
		$5, acc, $1),
      $4, -s,
      m4_ifelse($5, nom, $1,
		$5, gen, $1s,
		$5, dat, $1,
		$5, acc, $1),
      $4, -,
      m4_ifelse($5, nom, $1,
		$5, gen, $1s,
		$5, dat, $1,
		$5, acc, $1)),
   $6, pl,
   m4_ifelse(
      $4, -e,
      m4_ifelse($5, nom, $3,
		$5, gen, $3,
		$5, dat, $3n,
		$5, acc, $3),
      $4, -se,
      m4_ifelse($5, nom, $3,
		$5, gen, $3,
		$5, dat, $3n,
		$5, acc, $3),
      $4, -er,
      m4_ifelse($5, nom, $3,
		$5, gen, $3,
		$5, dat, $3n,
		$5, acc, $3),
      $4, -s,
      m4_ifelse($5, nom, $3,
		$5, gen, $3,
		$5, dat, $3,
		$5, acc, $3),
      $4, -,
      m4_ifelse(
	 $2, -n,
         m4_ifelse($5, nom, $3,
		   $5, gen, $3,
		   $5, dat, $3,
		   $5, acc, $3),
	 $2, -,
	 m4_ifelse($5, nom, $3,
		$5, gen, $3,
		$5, dat, $3n,
		$5, acc, $3))))')




//declineNN_strong_fem (nur fuer normal nouns NN)

m4_define(`declineNN_strong_fem',
`m4_ifelse(
   $6, sg,
   m4_ifelse(
      $4, -e,
      m4_ifelse($5, nom, $1,
		$5, gen, $1,
		$5, dat, $1,
		$5, acc, $1),
      $4, -se,
      m4_ifelse($5, nom, $1,
		$5, gen, $1,
		$5, dat, $1,
		$5, acc, $1),
      $4, -s,
      m4_ifelse($5, nom, $1,
		$5, gen, $1,
		$5, dat, $1,
		$5, acc, $1),
      $4, -,
      m4_ifelse($5, nom, $1,
		$5, gen, $1,
		$5, dat, $1,
		$5, acc, $1)),
   $6, pl,
   m4_ifelse(
      $4, -e,
      m4_ifelse($5, nom, $3,
		$5, gen, $3,
		$5, dat, $3n,
		$5, acc, $3),
      $4, -se,
      m4_ifelse($5, nom, $3,
		$5, gen, $3,
		$5, dat, $3n,
		$5, acc, $3),
      $4, -s,
      m4_ifelse($5, nom, $3,
		$5, gen, $3,
		$5, dat, $3,
		$5, acc, $3),
      $4, -,
      m4_ifelse($5, nom, $3,
		$5, gen, $3,
		$5, dat, $3n,
		$5, acc, $3)))')


//declineNN_weak_masc (nur fuer normal nouns NN)

m4_define(`declineNN_weak_masc',
`m4_ifelse(
   $6, sg,
   m4_ifelse(
      $4, -en,
      m4_ifelse($5, nom, $1,
		$5, gen, $1en,
		$5, dat, $1en,
		$5, acc, $1en),
      $4, -n,
      m4_ifelse($5, nom, $1,
		$5, gen, $1n,
		$5, dat, $1n,
		$5, acc, $1n)),
   $6, pl,
   m4_ifelse($5, nom, $3,
	     $5, gen, $3,
	     $5, dat, $3,
	     $5, acc, $3))')


//declineNN_weak_fem (nur fuer normal nouns NN)

m4_define(`declineNN_weak_fem',
`m4_ifelse(
   $6, sg,
   m4_ifelse($5, nom, $1,
	     $5, gen, $1,
	     $5, dat, $1,
	     $5, acc, $1),
   $6, pl,
   m4_ifelse($5, nom, $3,
	     $5, gen, $3,
	     $5, dat, $3,
	     $5, acc, $3))')


//declineNN_mixed_masc (nur fuer normal nouns NN)

m4_define(`declineNN_mixed_masc',
`m4_ifelse(
   $6, sg,
   m4_ifelse(
      $4, -en,
      m4_ifelse($5, nom, $1,
		$5, gen, $1es,
		$5, dat, $1,
		$5, acc, $1),
      $4, -n,
      m4_ifelse($5, nom, $1,
		$5, gen, $1s,
		$5, dat, $1,
		$5, acc, $1)),
   $6, pl,
   m4_ifelse($5, nom, $3,
	     $5, gen, $3,
	     $5, dat, $3,
	     $5, acc, $3))')


//declineNN_mixed_neut (nur fuer normal nouns NN)

m4_define(`declineNN_mixed_neut',
`m4_ifelse(
   $6, sg,
   m4_ifelse(
      $4, -en,
      m4_ifelse($5, nom, $1,
		$5, gen, $1es,
		$5, dat, $1,
		$5, acc, $1),
      $4, -n,
      m4_ifelse($5, nom, $1,
		$5, gen, $1s,
		$5, dat, $1,
		$5, acc, $1)),
   $6, pl,
   m4_ifelse($5, nom, $3,
	     $5, gen, $3,
	     $5, dat, $3,
	     $5, acc, $3))')

//MakeWeekday und MakeMonthName

m4_define( MakeWeekday, `
$1 := 
[ cat: NN, 
  person: 3, 
  gender: masc, 
  case: #1( nom | dat | acc ), 
  number: sg, 
  countable: yes, 
  sort: weekday,
  timeunit:yes,
  args:
  [ 1:
    [ 
      realized:yes,
      obl:0.5,
      cat:Artikel,
      case: #1(nom|dat|acc), 
      gender: masc,
      number: sg
    ]
  ]
];'

)

m4_define( MakeMonthName, `
$1 := 
[ cat: NN, 
  person: 3, 
  gender: masc, 
  case: #1( nom | dat | acc ), 
  number: sg, 
  countable: yes, 
  sort: month,
  timeunit:yes,
  args:
  [ 1:
    [ realized:yes,
      obl:0.5,
      cat:Artikel,
      case: #1(nom|dat|acc), 
      gender: masc,
      number: sg
    ]
  ]
];

')


//////////////////////////////////////////////////////////////////////////////
// Kategorie PPOSAT
//////////////////////////////////////////////////////////////////////////////

//MakePPOSAT
// $1:	Zu deklinierendes Wort
// $2:	person (1 o. 2 o. 3)
// $3:	Numerus (sg o. pl)
// $4: 	definite (yes o. no)

m4_define(`MakePPOSAT',
MakeOnePPOSAT(declinePPOSAT($1, masc, $3, nom), $3, masc, nom, $2, $4);
MakeOnePPOSAT(declinePPOSAT($1, masc, $3, gen), $3, masc, gen, $2, $4);
MakeOnePPOSAT(declinePPOSAT($1, masc, $3, dat), $3, masc, dat, $2, $4);
MakeOnePPOSAT(declinePPOSAT($1, masc, $3, acc), $3, masc, acc, $2, $4);
MakeOnePPOSAT(declinePPOSAT($1, fem, $3, nom), $3, fem, nom, $2, $4);
MakeOnePPOSAT(declinePPOSAT($1, fem, $3, gen), $3, fem, gen, $2, $4);
MakeOnePPOSAT(declinePPOSAT($1, fem, $3, dat), $3, fem, dat, $2, $4);
MakeOnePPOSAT(declinePPOSAT($1, fem, $3, acc), $3, fem, acc, $2, $4);
MakeOnePPOSAT(declinePPOSAT($1, neut, $3, nom), $3, neut, nom, $2, $4);
MakeOnePPOSAT(declinePPOSAT($1, neut, $3, gen), $3, neut, gen, $2, $4);
MakeOnePPOSAT(declinePPOSAT($1, neut, $3, dat), $3, neut, dat, $2, $4);
MakeOnePPOSAT(declinePPOSAT($1, neut, $3, acc), $3, neut, acc, $2, $4);
)


//MakeOnePPOSAT
// $1:	Dekliniertes Wort
// $2:  Numerus
// $3:	Genus
// $4:	Kasus
// $5:	person (1 o. 2 o. 3)
// $6:	definite (yes o. no)

m4_define(`MakeOnePPOSAT',
`$1 :=
[ cat:PPOSAT,
  person:$5,
  case:$4,
  number:$2,
  gender:$3,
  definite:$6
]')



m4_define(`declinePPOSAT',
`m4_ifelse(
   $3, sg,
   m4_ifelse(
      $2, masc,
      m4_ifelse($4, nom, $1,
		$4, gen, $1es,
		$4, dat, $1em,
		$4, acc, $1en),
      $2, fem,
      m4_ifelse($4, nom, $1e,
		$4, gen, $1er,
		$4, dat, $1er,
		$4, acc, $1e),
      $2, neut,
      m4_ifelse($4, nom, $1,
		$4, gen, $1es,
		$4, dat, $1em,
		$4, acc, $1)),
   $3, pl,
   m4_ifelse(
      $2, masc,
      m4_ifelse($4, nom, $1e,
		$4, gen, $1er,
		$4, dat, $1en,
		$4, acc, $1e),
      $2, fem,
      m4_ifelse($4, nom, $1e,
		$4, gen, $1er,
		$4, dat, $1en,
		$4, acc, $1e),
      $2, neut,
      m4_ifelse($4, nom, $1e,
		$4, gen, $1er,
		$4, dat, $1en,
		$4, acc, $1e)))')


//////////////////////////////////////////////////////////////////////////////
// Kategorien ADJA und ADJP
//////////////////////////////////////////////////////////////////////////////

//MakeADJ
//
// Als Parameter $1 bis $3 sind die attributiven Formen des Adjektivs zu
// bergeben (beachte aber die besondere Form fr $3, siehe Datei ADJ01.m4).
// Hieraus werden desweiteren noch die entsprechenden prdikativen Formen
// erzeugt.
//
// $1:	Adjektiv attributiv Positiv
// $2:	Adjektiv attributiv Komparativ
// $3: 	Adjektiv attributiv Superlativ
//
// $4:  sort (fuer praedikativen Gebrauch)
// $5:  Dieses Argument wird unverndert bernommen (nur prdikativer Gebrauch)


m4_define(`MakeADJ',
`m4_ifelse($1,`',,`MakeAdjektiv($1,,grade:positive)')
m4_ifelse($2,`',,`MakeAdjektiv($2,,grade:comparative)')
m4_ifelse($3,`',,`MakeAdjektiv($3,,grade:superlative)')
m4_ifelse($1,`',,`MakeADJP($1, $4, positive, $5)')
m4_ifelse($2,`',,`MakeADJP($2, $4, comparative, $5)')
m4_ifelse($3,`',,`MakeADJP($3en, $4, superlative, $5)')
')


m4_define(`MakeADJP',
`m4_ifelse($4,`',
`$1 :=
[ cat:ADJP,
  person:3,
  grade:$3,
  case:nom,
  sort:$2
];',
`$1 :=
[ cat:ADJP,
  person:3,
  grade:$3,
  case:nom,
  sort:$2,
  nshift(3,$@)
];')')


m4_define(`MakeNumberADJ',
`MakeAdjektiv($1,,value:$2)')

//MakeADJspecial
//
// Als Parameter $1 bis $3 sind die attributiven Formen des Adjektivs zu
// bergeben (beachte aber die besondere Form fr $3, siehe Datei ADJ01.m4).
// Parameter $4 bis $6: prdikative Form des Adjektivs (Positiv, Komparativ,
// Superlativ).
//
// $1:	Adjektiv attributiv Positiv
// $2:	Adjektiv attributiv Komparativ
// $3: 	Adjektiv attributiv Superlativ
// $4: 	Adjektiv prdikativ Positiv
// $5:  Adjektiv prdikativ Komparativ
// $6:	Adjektiv prdikativ Superlativ
// $7:  sort (fuer praedikativen Gebrauch)
// $8:  Dieses Argument wird unverndert bernommen (nur prdikativer Gebrauch)


m4_define(`MakeADJspecial',
`m4_ifelse($1,`',,`MakeAdjektiv($1,,grade:positive)')
m4_ifelse($2,`',,`MakeAdjektiv($2,,grade:comparative)')
m4_ifelse($3,`',,`MakeAdjektiv($3,,grade:superlative)')
m4_ifelse($4,`',,`MakeADJP($4, $7, positive, $8)')
m4_ifelse($5,`',,`MakeADJP($5, $7, comparative, $8)')
m4_ifelse($6,`',,`MakeADJP($6, $7, superlative, $8)')
')


//////////////////////////////////////////////////////////////////////////////
// Deklination
//////////////////////////////////////////////////////////////////////////////
// Decline(STAMM,D,G,N,K)
//
//   STAMM: Verbstamm
//   D    : Deklination strong,weak,mixed
//   N    : Numerus
//   K    : Kasus


m4_define(`Decline',
`m4_ifelse(
   $2, strong,
   m4_ifelse(
      $4, sg,
      m4_ifelse(
	 $3, masc,
	 m4_ifelse($5, nom, $1er,
		$5, gen, $1en,
		$5, dat, $1em,
		$5, acc, $1en),
	 $3, fem,
	 m4_ifelse($5, nom, $1e,
		$5, gen, $1er,
		$5, dat, $1er,
		$5, acc, $1e),
	 $3, neut,
	 m4_ifelse($5, nom, $1es,
		$5, gen, $1en,
		$5, dat, $1em,
		$5, acc, $1es)),
      $4, pl, 
      m4_ifelse($5, nom, $1e,
	     $5, gen, $1er,
	     $5, dat, $1en,
	     $5, acc, $1e)),
   $2, weak,
   m4_ifelse(
      $4, sg,
      m4_ifelse(
	 $3, masc,
	 m4_ifelse($5, nom, $1e,
		$5, gen, $1en,
		$5, dat, $1en,
		$5, acc, $1en),
	 $3, fem,
	 m4_ifelse($5, nom, $1e,
		$5, gen, $1en,
		$5, dat, $1en,
		$5, acc, $1e),
	 $3, neut,
	 m4_ifelse($5, nom, $1e,
		$5, gen, $1en,
		$5, dat, $1en,
		$5, acc, $1e)),
      $4, pl, $1en),
   $2, mixed,
   m4_ifelse(
      $4, sg,
      m4_ifelse(
	 $3, masc,
	 m4_ifelse($5, nom, $1er,
		$5, gen, $1en,
		$5, dat, $1en,
		$5, acc, $1en),
	 $3, fem,
	 m4_ifelse($5, nom, $1e,
		$5, gen, $1en,
		$5, dat, $1en,
		$5, acc, $1e),
	 $3, neut,
	 m4_ifelse($5, nom, $1es,
		$5, gen, $1en,
		$5, dat, $1en,
		$5, acc, $1es)),
      $4, pl, $1en))')



//////////////////////////////////////////////////////////////////////////////
// Erzeugen von finiten Verben
//////////////////////////////////////////////////////////////////////////////
//   MakeVerb(INF,ID,S1,S2,S3,P1,P2,P3,...)
//
//   INF               : Infinitiv
//   ID                : freier Lexikonpr"afix
//   S1,S2,S3          : 1.,2.,3. Person Singular
//   P1,P2,P3          : 1.,2.,3. Person Plural
//   ...               : sonstige Features

// Ein Verb
m4_define(`MakeOneVerb',
`$3 := [
   baseform:$1,
   number:$4, 
   person:$5,
   m4_patsubst(`m4_patsubst(`nshift(5,$@)',`m4_person',`person:$5')',`m4_number',`number:$4')]')

// Alle Verben
m4_define(`MakeVerb', 
`m4_pushdef(`_tmp',`nshift(8,$@)')
m4_ifelse($3,`',,1,1,`MakeOneVerb($1,$2,$3,sg,1,_tmp)';)
m4_ifelse($4,`',,1,1,`MakeOneVerb($1,$2,$4,sg,2,_tmp)';)
m4_ifelse($5,`',,1,1,`MakeOneVerb($1,$2,$5,sg,3,_tmp)';)
m4_ifelse($6,`',,1,1,`MakeOneVerb($1,$2,$6,pl,1,_tmp)';)
m4_ifelse($7,`',,1,1,`MakeOneVerb($1,$2,$7,pl,2,_tmp)';)
m4_ifelse($8,`',,1,1,`MakeOneVerb($1,$2,$8,pl,3,_tmp)';)
m4_popdef(`_tmp')')


//////////////////////////////////////////////////////////////////////////////
// Erzeugen aller Verben
//////////////////////////////////////////////////////////////////////////////
//   MakeAllVerb(INF,ID,PI,II,K2,...)
//
//   INF               : Infinitiv
//   ID                : freier Lexikonprfix
//   PI                : Prsens Indikativ
//   II                : Imperfekt Indikativ
//   K2                : Prteritum Konjunktiv 2
//   ...               : variable Features

m4_define(MakeAllVerbs, 
`m4_pushdef(`_tmp',`nshift(5,$@)')
m4_ifelse(`$3',`',,1,1,
`// $1 Prsens Indikativ
MakeVerb($1,$2_pres_ind,$3,
   tense:present,
   mood:indicative,
   _tmp)')
m4_ifelse(`$5',`*',
`// $1 Imperfekt+Indikativ=Prteritum+Konjunktiv
MakeVerb($1,$2,$4,
   tense:#1(past|present),
   mood:#1(indicative|subjunctive),
   _tmp)',1,1,
`m4_ifelse(`$4',`',,1,1,
`// $1 Imperfekt Indikativ
MakeVerb($1,$2_past_ind,$4,
   tense:past,
   mood:indicative,
   _tmp)')
m4_ifelse(`$5',`',,1,1,
`// $1 Prteritum Konjunktiv 2
MakeVerb($1,$2_pres_sub,$5,
   tense:present,
   mood:subjunctive,
   _tmp)')')
m4_popdef(`_tmp')m4_dnl')

//////////////////////////////////////////////////////////////////////////////
// Erzeugt alle Eintrge von FIN sowie INF fr nicht trennbare Verben
//////////////////////////////////////////////////////////////////////////////

m4_define(MakeNonseperableVerbs,
`MakeAllVerbs(`$1',`$2',`$3',`$4',`$5',
`cat:FIN, sort:$6,'
   m4_ifelse(`$7',`',,`loctype:$7'`,')
   `args:
    [' 
      $8
      $9
      $10
      $11
   `]'
)'
$1 :=
[ `cat:INF,
  baseform:$1,
  sort:$6,
  m4_ifelse(`$7',`',,`loctype:$7'`,')
   args:
   [
    m4_ifelse(`$8',`',,`m4_patsubst(`m4_patsubst(`m4_patsubst(
                          `$8', ` m4_[a-z]*,')',
                          `obl:[0-9].[0-9]*', `obl:0.0')',
                          `realized:yes', `realized:no')')
    m4_ifelse(`$9',`',,`m4_patsubst(`$9', ` m4_[a-z]*,')')
    m4_ifelse(`$10',`',,`m4_patsubst(`$10', ` m4_[a-z]*,')')
    m4_ifelse(`$11',`',,`m4_patsubst(`$11', ` m4_[a-z]*,')')
   ]'
];
)


//////////////////////////////////////////////////////////////////////////////
// Erzeugen der Kategorien VMODFIN und VMODINF
//////////////////////////////////////////////////////////////////////////////

m4_define(MakeVMOD,
`MakeAllVerbs($1,_MOD, `$2', `$3', `$4',
    `cat:VMODFIN,
    sort:state,
    args:
    [ 1:
       [ realized:yes, obl:0.1,
         cat:Nominal,
         case:nom,
         m4_person,
         m4_number,
         sort:top
       ],
      2:
       [ realized:yes, obl:0.1,
         cat:infinit,
         sort:top
       ]
    ]'
  )'
$1 :=
[ `cat:VMODINF,
  baseform:$1,
  sort:state,
  args:
  [ 2:
    [ realized:yes, obl:0.1,
      cat:infinit,
      sort:top
    ]
  ]'
];
)



//////////////////////////////////////////////////////////////////////////////
// Erzeugen von Partizipien
//////////////////////////////////////////////////////////////////////////////
//   MakePartizipPraesens(STAMM,ID, ...);
//
//   STAMM       : Stammform
//   ID          : freier Prfix an Lexikon-Id
//   ...         : sonstige Features

// Ein Partizip
m4_define(`MakeOnePartizip',
`$1 := [
   cat:ADJA, 
   case:$2, 
   number:$3, 
   gender:$4, 
   tense:present,
   flexion:$5 m4_ifelse($7,`',,1,1,`,
   nshift(6, $@)')m4_dnl
]')

// Alle Partizipien
m4_define(`MakePartizipPraesens',`
m4_pushdef(`_tmp',`nshift(2,$@)')
MakeOnePartizip(
   Decline($1,strong,masc,sg,nom),nom,sg,masc,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,masc,sg,acc),acc,sg,masc,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,masc,sg,dat),dat,sg,masc,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,masc,sg,gen),gen,sg,masc,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,masc,pl,nom),nom,pl,masc,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,masc,pl,acc),acc,pl,masc,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,masc,pl,dat),dat,pl,masc,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,masc,pl,gen),gen,pl,masc,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,fem,sg,nom),nom,sg,fem,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,fem,sg,acc),acc,sg,fem,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,fem,sg,dat),dat,sg,fem,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,fem,sg,gen),gen,sg,fem,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,fem,pl,nom),nom,pl,fem,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,fem,pl,acc),acc,pl,fem,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,fem,pl,dat),dat,pl,fem,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,fem,pl,gen),gen,pl,fem,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,neut,sg,nom),nom,sg,neut,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,neut,sg,acc),acc,sg,neut,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,neut,sg,dat),dat,sg,neut,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,neut,sg,gen),gen,sg,neut,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,neut,pl,nom),nom,pl,neut,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,neut,pl,acc),acc,pl,neut,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,neut,pl,dat),dat,pl,neut,strong,$2,_tmp);
MakeOnePartizip(
   Decline($1,strong,neut,pl,gen),gen,pl,neut,strong,$2,_tmp);

MakeOnePartizip(
   Decline($1,weak,masc,sg,nom),nom,sg,masc,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,masc,sg,acc),acc,sg,masc,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,masc,sg,dat),dat,sg,masc,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,masc,sg,gen),gen,sg,masc,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,masc,pl,nom),nom,pl,masc,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,masc,pl,acc),acc,pl,masc,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,masc,pl,dat),dat,pl,masc,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,masc,pl,gen),gen,pl,masc,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,fem,sg,nom),nom,sg,fem,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,fem,sg,acc),acc,sg,fem,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,fem,sg,dat),dat,sg,fem,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,fem,sg,gen),gen,sg,fem,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,fem,pl,nom),nom,pl,fem,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,fem,pl,acc),acc,pl,fem,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,fem,pl,dat),dat,pl,fem,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,fem,pl,gen),gen,pl,fem,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,neut,sg,nom),nom,sg,neut,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,neut,sg,acc),acc,sg,neut,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,neut,sg,dat),dat,sg,neut,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,neut,sg,gen),gen,sg,neut,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,neut,pl,nom),nom,pl,neut,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,neut,pl,acc),acc,pl,neut,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,neut,pl,dat),dat,pl,neut,weak,$2,_tmp);
MakeOnePartizip(
   Decline($1,weak,neut,pl,gen),gen,pl,neut,weak,$2,_tmp);

MakeOnePartizip(
   Decline($1,mixed,masc,sg,nom),nom,sg,masc,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,masc,sg,acc),acc,sg,masc,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,masc,sg,dat),dat,sg,masc,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,masc,sg,gen),gen,sg,masc,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,masc,pl,nom),nom,pl,masc,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,masc,pl,acc),acc,pl,masc,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,masc,pl,dat),dat,pl,masc,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,masc,pl,gen),gen,pl,masc,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,fem,sg,nom),nom,sg,fem,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,fem,sg,acc),acc,sg,fem,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,fem,sg,dat),dat,sg,fem,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,fem,sg,gen),gen,sg,fem,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,fem,pl,nom),nom,pl,fem,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,fem,pl,acc),acc,pl,fem,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,fem,pl,dat),dat,pl,fem,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,fem,pl,gen),gen,pl,fem,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,neut,sg,nom),nom,sg,neut,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,neut,sg,acc),acc,sg,neut,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,neut,sg,dat),dat,sg,neut,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,neut,sg,gen),gen,sg,neut,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,neut,pl,nom),nom,pl,neut,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,neut,pl,acc),acc,pl,neut,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,neut,pl,dat),dat,pl,neut,mixed,$2,_tmp);
MakeOnePartizip(
   Decline($1,mixed,neut,pl,gen),gen,pl,neut,mixed,$2,_tmp)
m4_popdef(`_tmp')
')

//////////////////////////////////////////////////////////////////////////////
// Erzeugen von Adjektiven
//////////////////////////////////////////////////////////////////////////////
//   MakeAdjektiv(STAMM,ID, ...);
//   
//   STAMM       : Stammform
//   ID          : freier Prfix an Lexikon-Id
//   ...         : sonstige Features

// Ein Adjektiv
m4_define(`MakeOneAdj',
`$1 := [
   cat:ADJA, 
   case:$2, 
   number:$3, 
   gender:$4, 
   flexion:$5 m4_ifelse($7,`',,1,1,`,
   nshift(6, $@)')m4_dnl
]')

// Alle Adjektive
m4_define(`MakeAdjektiv',`
m4_pushdef(`_tmp',`nshift(2,$@)')
//Decline($1,strong,masc,sg,nom)
MakeOneAdj(
   Decline($1,strong,masc,sg,nom),nom,sg,masc,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,masc,sg,acc),acc,sg,masc,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,masc,sg,dat),dat,sg,masc,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,masc,sg,gen),gen,sg,masc,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,fem,sg,nom),nom,sg,fem,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,fem,sg,acc),acc,sg,fem,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,fem,sg,dat),dat,sg,fem,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,fem,sg,gen),gen,sg,fem,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,neut,sg,nom),nom,sg,neut,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,neut,sg,acc),acc,sg,neut,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,neut,sg,dat),dat,sg,neut,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,neut,sg,gen),gen,sg,neut,strong,$2,_tmp);

MakeOneAdj(
   Decline($1,strong,bot,pl,nom),nom,pl,bot,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,bot,pl,acc),acc,pl,bot,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,bot,pl,dat),dat,pl,bot,strong,$2,_tmp);
MakeOneAdj(
   Decline($1,strong,bot,pl,gen),gen,pl,bot,strong,$2,_tmp);


MakeOneAdj(
   Decline($1,weak,masc,sg,nom),nom,sg,masc,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,masc,sg,acc),acc,sg,masc,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,masc,sg,dat),dat,sg,masc,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,masc,sg,gen),gen,sg,masc,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,fem,sg,nom),nom,sg,fem,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,fem,sg,acc),acc,sg,fem,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,fem,sg,dat),dat,sg,fem,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,fem,sg,gen),gen,sg,fem,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,neut,sg,nom),nom,sg,neut,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,neut,sg,acc),acc,sg,neut,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,neut,sg,dat),dat,sg,neut,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,neut,sg,gen),gen,sg,neut,weak,$2,_tmp);

MakeOneAdj(
   Decline($1,weak,bot,pl,nom),nom,pl,bot,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,bot,pl,acc),acc,pl,bot,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,bot,pl,dat),dat,pl,bot,weak,$2,_tmp);
MakeOneAdj(
   Decline($1,weak,bot,pl,gen),gen,pl,bot,weak,$2,_tmp);


MakeOneAdj(
   Decline($1,mixed,masc,sg,nom),nom,sg,masc,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,masc,sg,acc),acc,sg,masc,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,masc,sg,dat),dat,sg,masc,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,masc,sg,gen),gen,sg,masc,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,fem,sg,nom),nom,sg,fem,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,fem,sg,acc),acc,sg,fem,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,fem,sg,dat),dat,sg,fem,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,fem,sg,gen),gen,sg,fem,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,neut,sg,nom),nom,sg,neut,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,neut,sg,acc),acc,sg,neut,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,neut,sg,dat),dat,sg,neut,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,neut,sg,gen),gen,sg,neut,mixed,$2,_tmp);

MakeOneAdj(
   Decline($1,mixed,bot,pl,nom),nom,pl,bot,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,bot,pl,acc),acc,pl,bot,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,bot,pl,dat),dat,pl,bot,mixed,$2,_tmp);
MakeOneAdj(
   Decline($1,mixed,bot,pl,gen),gen,pl,bot,mixed,$2,_tmp);
m4_popdef(`_tmp')
')



//////////////////////////////////////////////////////////////////////////////
// Erzeugen von Relativpronomen
//////////////////////////////////////////////////////////////////////////////
//   MakePREL(form, case, number, gender)
//   
//   form	: Wortform
//   case	: Kasus
//   number	: Numerus
//   gender	: Genus
m4_define( MakePREL, `

$1 :=
[ cat:PREL, 
  case:$2, 
  number:$3,
  gender:$4, 
  person:3,
  sort:bot,
  args:
  [ 4:
    [ realized:yes,
      obl:0.0,
      cat:Nominal,
      number:$3,
      gender:$4
    ]     
  ]
];

')

//////////////////////////////////////////////////////////////////////////////
// Erzeugen von Demonstrativpronomen
//////////////////////////////////////////////////////////////////////////////
//   MakePROS(form, case, number, gender)
//   
//   form	: Wortform
//   case	: Kasus
//   number	: Numerus
//   gender	: Genus


m4_define( MakePROS, `

$1 :=
[ cat:PROS, 
  sort:bot,
  person:3,
  case:$2,
  number:$3,
   gender:$4
];

')


') 

// end Makros
/////////////////////////////////////////////////////////////////////////////
