/*
 * File: levelvalue.h
 * Author: Ingo Schroeder
 * Purpose: header file for level values
 * Birth: 24/4/97
 *
 * $Id: levelvalue.h,v 1.35 2003/09/29 08:37:17 foth Exp $
 */

#ifndef LEVELVALUE_H
#define LEVELVALUE_H

/* ---------------------------------------------------------------------- */
#include <blah.h>
#include "input.h"
#include "lexemgraph.h"

/* ---------------------------------------------------------------------- */

#ifdef SWIG
typedef LevelValueStruct *LevelValue;
#else
typedef struct {
  int no;		     /* id used to find scores in cache */
  LexemNode modifier;	     /* modifying node */
  List modifiers;	     /* modifying nodes */
  Level level;	     /* level of modification relation */
  String label;		     /* label of modification relation */
  LexemNode modifiee;	     /* modified node */
  List modifiees;	     /* modified nodes */

  double score;		     /* score */
  List constraints;	     /* list of violated constraints */
  double limit;		     /* maximal attainable score of solution */
  Boolean isDeleted;	     /* flags to indicate deletion */
  int indexWRTNet;
} LevelValueStruct;
typedef LevelValueStruct *LevelValue;
#endif

/* ---------------------------------------------------------------------- */
#ifndef SWIG
#define NONSPEC ((Pointer)(-1))
#define NNONSPEC ((Pointer)(-2))
#define root(a) ((int)a == 0)
#define spec(a) ((int)a > 0)
#define nonspec(a) ((int)a == -1)
#define nnonspec(a) ((int)a == -2)
#endif

/* ---------------------------------------------------------------------- */

#ifndef SWIG
extern LevelValue lvNew(List modifiers, Level level,
			String label, List modifees);
extern void lvDelete(LevelValue lv);
extern LevelValue lvSet(LevelValue lv, List modifiers, Level level,
			String label, List modifiees);
extern Boolean lvCompatible(LexemGraph lg, LevelValue a, LevelValue b);
extern void lvPrint(unsigned long mode, LevelValue lv, BitVector isDeleted);
extern Connexion getCon(LevelValue a, LevelValue b);
extern Direction getDir(LevelValue x);
extern Boolean matchUnarySignature(LevelValue lv, Signature x);
extern Boolean matchBinarySignature(LevelValue lva, LevelValue lvb,
				    Signature x);
extern Boolean lvCompare(LevelValue a, LevelValue b);
extern Boolean lvCompareByLimit(LevelValue a, LevelValue b);
extern Boolean lvCompareByNaturalOrder(LevelValue a, LevelValue b);
extern LevelValue lvClone(LevelValue lv);
extern Boolean lvVectorCompatible(LevelValue lv, BitVector taboo);
extern void lvCheck(LevelValue lv);
extern Boolean lvTestAllDeleted(LevelValue lv, BitVector isDeleted);
#endif
extern int lvIndex(LevelValue lv);

/* ---------------------------------------------------------------------- */
#endif /* don't insert anything after this #endif */
