/*
 * File: lexemgraph.h
 * Author: Ingo Schroeder
 * Purpose: header file for lexem graph structures
 * Birth: 4/3/97
 *
 * $Id: lexemgraph.h,v 1.64 2004/01/13 13:26:10 foth Exp $
 */

#ifndef LEXEMGRAPH_H
#define LEXEMGRAPH_H

/* ---------------------------------------------------------------------- */
#include <blah.h>
#include "input.h"

/* ---------------------------------------------------------------------- */

/* forward declaration */
struct ChunkStruct;

#ifndef SWIG
typedef struct {
  Lattice lattice;                      /* underlying word graph */
  Vector graphemnodes;                  /* vector of grapheme nodes */
  Vector nodes;                         /* vector of lexeme nodes */
  int min;                              /* minimum start position */
  int max;                              /* maximum end position */
  /* the following parts are considered dynamic */
  Array distance;                       /* matrix of distances */
  long long *noOfPathsFromStart;        /* vector of numbers of paths */
  long long *noOfPathsToEnd;            /* vector of numbers of paths */
  long long noOfPaths;                  /* total number of paths through graph */
  BitVector isDeletedNode;              /* vector of boolean flags */
  Vector tags;
  List chunks;				/* set of all chunks of the lattice */
} LexemGraphStruct;
typedef LexemGraphStruct *LexemGraph;

typedef struct {
  int no;                               /* index in lg->graphemnodes */
  LexemGraph lexemgraph;                /* pointer back to the lexem graph */
  Arc arc;                              /* arc in word lattice */
  List lexemes;                         /* list of disambiguated lexeme nodes */
  Boolean live;                         /* are there any lexemes left undeleted? */
  struct ChunkStruct *chunk;            /* to which chunk do we belong */
} GraphemNodeStruct;
typedef GraphemNodeStruct *GraphemNode;
#else
typedef struct LexemGraphStruct *LexemGraph;
#endif

typedef struct {
  int no;                              /* index in LexemGraph::nodes */
  LexemGraph lexemgraph;               /* pointer back to the lexem graph */
  Arc arc;                             /* arc in word lattice */
  LexiconItem lexem;                  /* lexical entry */
  GraphemNode grapheme;                /* pointer to the original grapheme */
  Number tagscore;		       /* Tagger: probability of the associated category */
  Number limit;                        /* limit calculated during frobbing */
} LexemNodeStruct;
typedef LexemNodeStruct *LexemNode;

/* ---------------------------------------------------------------------- */
extern LexemGraph lgNew(Lattice lat);

#ifndef SWIG
/*
  This variable controls whether the levelvalues should be deflated if
  they are equivalent. Usually, we want this switched on, only for
  testing a value of FALSE might be appropriate.
*/
extern Boolean lgCompactLVs;

/* ---------------------------------------------------------------------- */

extern List lgPartitions(GraphemNode gn, BitString features);
extern long long computeNoOfPathsFromStart(LexemGraph lg, GraphemNode gn,
                                           long long sofar, long long maximal);
extern long long computeNoOfPathsToEnd(LexemGraph lg, GraphemNode gn,
                                 long long sofar, long long maximal);
extern void lgComputeDistances(LexemGraph lg);
extern void lgComputeNoOfPaths(LexemGraph lg);

extern LexemGraph lgNewInit();
extern Boolean lgNewIter(LexemGraph lg, Arc arc);
extern Boolean lgNewFinal(LexemGraph lg);

extern void lgPrint(unsigned long, LexemGraph lg);

extern Boolean lgNodeFollowsDirectly(GraphemNode a, GraphemNode b);
extern Boolean lgIsStartNode(GraphemNode n);
extern Boolean lgIsEndNode(GraphemNode n);

extern Boolean lgCompatibleNodes(LexemGraph lg, LexemNode a, LexemNode b);
extern int lgDistanceOfNodes(LexemGraph lg, LexemNode a, LexemNode b);
extern Boolean lgIsDeletedNode(LexemGraph lg, LexemNode n);
extern void lgDeleteNode(LexemGraph lg, LexemNode n);

extern void lgDelete(LexemGraph lg);

extern Boolean lgAreDeletedNodes( LexemGraph lg, List lexemes );
extern Boolean lgAreDeletableNodes( LexemGraph lg, List lexemes );
extern Boolean lgCompatibleSets( LexemGraph lg, List a, List b );
extern Boolean lgForbiddenBy( LexemGraph lg, LexemNode a, List b );
extern Boolean lgMayModify( LexemGraph lg, GraphemNode down, GraphemNode up );
extern Boolean lgMember( LexemNode ln, List lexemes );
extern Boolean lgSubset( List a, List b );
extern Boolean lgIntersectingSets( List a, List b );
extern void lgDeleteNodes( LexemGraph lg, List nodes );
extern int lgWidth (LexemGraph lg);
extern void lgPrintNode(unsigned long mode, LexemNode ln);
extern List lgMakePath(LexemGraph lg, List nodes);
extern Boolean lgOverlap(LexemNode a, LexemNode b);
extern void lgRequireLexeme(LexemGraph lg, BitVector v, LexemNode ln);
extern void lgRequireLexemes(LexemGraph lg, BitVector v, List which);
extern void lgCopyTagScores(LexemGraph destination, LexemGraph source);
extern Boolean lgUpdateArcs(LexemGraph lg, Lattice lat, List listArcs);
extern Boolean lgLexemeInLexemNodeList(LexiconItem le, List list);
extern Boolean lgCopySelection(LexemGraph destination, LexemGraph source);
extern List lgQueryCat(LexemGraph lg, Arc arc);
extern Boolean lgContains(LexemGraph lg, String form);

extern void lgInitialize();
extern List lgMostProbablePath(LexemGraph lg);
extern LexemGraph lgClone(LexemGraph lg);
#endif

/* ---------------------------------------------------------------------- */

#endif /* don't insert anything after this #endif */
