/* Copyright (C) 1997-2003 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without 
 * modifications, as long as this notice is preserved.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Purpose: Bison file for CDG parser
 * Author: Ingo Schroeder (see AUTHORS and THANKS for more)
 * Birth: 7/2/97
 * $Id: parser.y,v 1.83 2004/01/13 13:26:10 foth Exp $
 */

/* ---------------------------------------------------------------------- */
/* C declarations */
%{
#include <configure.h>
#include "my_libgen.h"
  
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <regex.h>
#include <blah.h>
#include "hook.h"
#include "input.h"
#include "parsing.h"
#include "predicates.h"
#include "functions.h"

/*
#define YYDEBUG 1
#define YYERROR_VERBOSE
*/
/* --- FORWARD DECLARATION ----------------------------- */
/* ----------------------------------------------------- */

Input parseResult = (Input)NULL;
int parseNoOfErrors = 0;
int parseNoOfWarnings = 0;

List variables = (List)NULL;
static Connexion theConnexion;
int timepoint = 0;
int arcCounter = 0;
int hierarchyDepth = 0;
int levelCounter = 0;
Boolean is_context_sensitive;
Hashtable current_map = NULL;
 
#define BUFFER_LIMIT 550
/* 
 * This is used only for converting integers to strings, 
 * i.e. 2^64 (==18446744073709551616) should be the upper limit.
 * 
 * 550 - schould be fine for path + filename + integer
 */
static char buffer[BUFFER_LIMIT];

String lastValidFound = NULL;
int lastValidLineNo = -1;

void yyLastValidFound( String s );
%}

/* ---------------------------------------------------------------------- */
/* Bison declarations */
%start InputSeq

%union {
  AnnoEntry ANNOENTRY;
  Annotation ANNOTATION;
  Arc ARC;
  AVNode AVNODE;
  Connexion CONNEXION;
  Constraint CONSTRAINT;
  Direction DIRECTION;
  Property PROPERTY;
  Formula FORMULA;
  FormulaType FORMULATYPE;
  Hierarchy HIERARCHY;
  Input INPUT;
  Lattice LATTICE;
  Level LEVELDECL;
  LexiconItem LEXICONITEM;
  List LIST;
  Number NUMBER;
  Sort SORT;
  Specification SPECIFICATION;
  String STRING;
  Subsumption SUBSUMPTION;
  Term TERM;
  Value VALUE;
  VarInfo VARINFO;
  List VARLIST;
}

%token CTRUE CFALSE
%token LABEL LEVEL
%token TBEGIN_LATTICE TEND_LATTICE
%token ISDEFINEDBY
%token MATCHES
%token MAP

%token CTOP 
%token CBOTTOM
%token CNONE
%token CINVERSE
%token CIDENT

%left BIIMPL
%left IMPL
%left '|'
%left '&' 
%left '~'

%nonassoc '=' '>' '<' GTEQ LSEQ NEQ LPMI

%left '+' '-'
%left '*' '/'

%token <NUMBER> NUM
%token <STRING> STR

%type <ANNOENTRY> AnnoEntry
%type <LIST> AnnoList
%type <ANNOTATION> Annotation
%type <ARC> Arc
%type <ARC> ArcVM
%type <LIST> ArcList
%type <LIST> ArcListVM
%type <AVNODE> AttributeValue
%type <LIST> Conjunction
%type <CONNEXION> Connexion
%type <CONSTRAINT> Constraint
%type <DIRECTION> Direction
%type <LIST> Disjunction
%type <FORMULA> Formula
%type <PROPERTY> Property
%type <LIST> Properties
%type <LIST> PropertyList
%type <HIERARCHY> Hierarchy
%type <VALUE> InfoString
%type <VALUE> InfoModule
%type <VALUE> InfoItem
%type <INPUT> InputSeq
%type <LIST> LabelList
%type <LEVELDECL> Level
%type <LEXICONITEM> LexiconItem
%type <LIST> Hashentry
%type <LIST> Hashentrylist
%type <CONSTRAINT> Parameter
%type <LIST> Path
%type <FORMULATYPE> RelationType
%type <SORT> Sort
%type <LIST> SortList
%type <SPECIFICATION> Specification
%type <LIST> SpecSeq
%type <LIST> Stringlist
%type <SUBSUMPTION> Subsumption
%type <LIST> SubsumptionList
%type <LIST> TypeSeq
%type <TERM> Term
%type <LIST> TermList
%type <VALUE> Value
%type <LIST> ValueList
%type <LATTICE> Lattice
%type <VARINFO> VarInfo
%type <VARLIST> VarList

%%
/* ---------------------------------------------------------------------- */
/* Grammar rules */

AnnoEntry:
    STR ':' STR BIIMPL AnnoList
    {
      yyLastValidFound( "annotation" );
      $$ = (AnnoEntry)memMalloc( sizeof(AnnoEntryStruct) );
      $$->id = $1;
      $$->lattice = $3;
      $$->annos = $5;
      $$->filename = parseCurrentFile;
      $$->lineNo = parseLineNo;
    }
;

AnnoList:
    AnnoList ',' Annotation
    {
      $$ = listAppendItem( $1, $3 );
    }
  | Annotation
    {
      $$ = listPrependItem( NULL, $1 );
    }
;

Annotation:
    NUM NUM STR SpecSeq
    {
      $$ = newAnnotation($1, $2, $3);
      $$->specs = $4;
    }
;
      
Arc:
  STR
    {
      yyLastValidFound( "arc" );
      $$ = newArc(timepoint, timepoint+1, arcCounter, $1, 0.0, NULL, 0, 0);
      arcCounter++;
      timepoint++;
    }
  | NUM NUM STR
    {
      yyLastValidFound( "arc" );
      $$ = newArc((int)$1,(int)$2, arcCounter, $3, 0.0, NULL, 0, 0);
      timepoint = max( timepoint, $$->to );
      arcCounter++;
    }
  | NUM NUM STR NUM
    {
      yyLastValidFound( "arc" );
      $$ = newArc((int)$1,(int)$2, arcCounter, $3, $4, NULL, 0, 0);
      timepoint = max( timepoint, $$->to );
      arcCounter++;
    }
;

ArcVM:
  NUM NUM STR NUM NUM NUM InfoString
    {
      yyLastValidFound( "VM arc" );
      $$ = newArc((int)$1,(int)$2, arcCounter,  $3, $4, $7, (int)$5, (int)$6);
      timepoint = max( timepoint, $$->to );
      arcCounter++;
    }
| NUM NUM '<' STR '>' NUM NUM NUM InfoString
    {
      yyLastValidFound( "VM arc" );
      $$ = newArc((int)$1,(int)$2, arcCounter, $4, $6, $9, (int)$7, (int)$8);
      timepoint = max( timepoint, $$->to );
      arcCounter++;
    } 
| NUM NUM STR NUM NUM NUM 
    {
      yyLastValidFound( "VM arc" );
      $$ = newArc((int)$1,(int)$2, arcCounter, $3, $4, NULL, (int)$5, (int)$6);
      timepoint = max( timepoint, $$->to );
      arcCounter++;
    }
| NUM NUM '<' STR '>' NUM NUM NUM 
    {
      yyLastValidFound( "VM arc" );
      $$ = newArc((int)$1,(int)$2, arcCounter, $4, $6, NULL, (int)$7, (int)$8);
      timepoint = max( timepoint, $$->to );
      arcCounter++;
    }
;

ArcList:
    ArcList ',' Arc
    {
      $$ = listAppendItem( $1, $3 );
    }
  | Arc
    {
      $$ = listPrependItem( NULL, $1 );
    }
;

ArcListVM:
  /* same as by ArcList, but without the "," */
    ArcListVM ArcVM
    {
      $$ = listAppendItem( $1, $2 );
    }
  | ArcVM 
    {
      $$ = listPrependItem( NULL, $1 );
    }
;

AttributeValue:
    STR ':' Value
    {
      $$ = (AVNode)memMalloc( sizeof(AVNodeStruct) );
      $$->attribute = $1;
      $$->value = $3;
    }
  | NUM ':' Value
    {
      $$ = (AVNode)memMalloc( sizeof(AVNodeStruct) );
      sprintf( buffer, "%d", (int)$1);
      $$->attribute = strRegister( buffer );
      $$->value = $3;
    }
;

Conjunction:
    Conjunction ',' Value
    {
      /* check for over-specified values */
      if($3->type == VTAVNode) {
	List l;
	String att = ((AVNode)$3->data.avnode)->attribute;
	Value v;
	
	for(l = $1; l != NULL; l = listNext(l)) {
	  v = listItem(l);
	  if(v->type == VTAVNode) {
	    if(!strcmp(((AVNode)v->data.avnode)->attribute, att)) {
	      yywarning("feature multiply specified");
	    }
	  } 
	}
      }
      $$ = listAppendItem( $1, $3 );
    }
  | Value
    {
      $$ = listPrependItem( NULL, $1 );
    }
;

Connexion:  
    ','
    {
      $$ = AnyCon; 
    }
  | 
    CTOP    
   {
      $$ = Top; 
   }
  | 
    CBOTTOM
    {
      $$ = Bottom; 
    }
  | 
    '/'
    {
      $$ = Over; 
    }
  | 
    '\\'
    {
      $$ = Under; 
    }
  | 
    CNONE
    {
      $$ = None;
    }
  | 
    CINVERSE
    {
      $$ = Inverse; 
    }
  | 
    CIDENT
    {
      $$ = Identical; 
    }
;    

Constraint:
    '{' VarList '}' ':' STR ':' STR ':' NUM ':' Formula
    { 
      List l;
      Section s;

      yyLastValidFound( "constraint" );
      $$ = (Constraint)memMalloc( sizeof(ConstraintStruct) );
      $$->vars = $2;
      $$->sig = (Signature)memMalloc( sizeof(SignatureStruct) );
      $$->sig->arity = listLength( $2 );
      $$->sig->conn = theConnexion;
      $$->id = $5;
      $$->penaltyTerm = NULL;
      $$->penalty = $9;
      $$->formula = $11;
      $$->active = TRUE;
      $$->filename = parseCurrentFile;
      $$->lineNo = parseLineNo;
      $$->cfunction = NULL;
      variables = NULL;

      for ( l = inputSections;
	    l != NULL && strcmp( ((Section)listItem(l))->id, $7 ) != 0;
	    l = listNext(l) )
	/* empty body */
	;
      if ( l == NULL ) {
	/* new section found */
	s = (Section)memMalloc( sizeof(SectionStruct) );
	s->id = $7;
	s->active = TRUE;
	s->counter = 1;
	inputSections = listAppendItem( inputSections, s );
	$$->section = s;
      } else {
	$$->section = (Section)listItem(l);
	$$->section->counter++;
	cdgFreeString( $7 );
      }
      $$->is_context_sensitive = is_context_sensitive;
      is_context_sensitive = FALSE;

    }
  | '{' VarList '}' ':' STR ':' STR ':' '[' Term ']' ':' Formula
    { 
      List l;
      Section s;

      yyLastValidFound( "constraint" );
      $$ = (Constraint)memMalloc( sizeof(ConstraintStruct) );
      $$->vars = $2;
      $$->sig = (Signature)memMalloc( sizeof(SignatureStruct) );
      $$->sig->arity = listLength( $2 );
      $$->sig->conn = theConnexion;
      $$->id = $5;
      $$->penaltyTerm = $10;
      $$->penalty = 0.0;
      $$->formula = $13;
      $$->active = TRUE;
      $$->filename = parseCurrentFile;
      $$->lineNo = parseLineNo;
      $$->cfunction = NULL;
      variables = NULL;

      for ( l = inputSections;
	    l != NULL && strcmp( ((Section)listItem(l))->id, $7 ) != 0;
	    l = listNext(l) )
	/* empty body */
	;
      if ( l == NULL ) {
	s = (Section)memMalloc( sizeof(SectionStruct) );
	s->id = $7;
	s->active = TRUE;
	s->counter = 1;
	inputSections = listAppendItem( inputSections, s );
	$$->section = s;
      } else {
	$$->section = (Section)listItem(l);
	$$->section->counter++;
	cdgFreeString( $7 );
      }
      $$->is_context_sensitive = is_context_sensitive;
      is_context_sensitive = FALSE;
    }
  | '{' VarList '}' ':' STR ':' NUM ':' Formula
    { 
      yyLastValidFound( "constraint" );
      $$ = (Constraint)memMalloc( sizeof(ConstraintStruct) );
      $$->vars = $2;
      $$->sig = (Signature)memMalloc( sizeof(SignatureStruct) );
      $$->sig->arity = listLength( $2 );
      $$->sig->conn = theConnexion;
      $$->id = $5;
      $$->section = inputDefaultSection;
      inputDefaultSection->counter++;
      $$->penaltyTerm = NULL;
      $$->penalty = $7;
      $$->formula = $9;
      $$->active = TRUE;
      $$->filename = parseCurrentFile;
      $$->lineNo = parseLineNo;
      $$->cfunction = NULL;
      variables = NULL;
      $$->is_context_sensitive = is_context_sensitive;
      is_context_sensitive = FALSE;
    }
  | '{' VarList '}' ':' STR ':' '[' Term ']' ':' Formula
    { 
      yyLastValidFound( "constraint" );
      $$ = (Constraint)memMalloc( sizeof(ConstraintStruct) );
      $$->vars = $2;
      $$->sig = (Signature)memMalloc( sizeof(SignatureStruct) );
      $$->sig->arity = listLength( $2 );
      $$->sig->conn = theConnexion;
      $$->id = $5;
      $$->section = inputDefaultSection;
      inputDefaultSection->counter++;
      $$->penaltyTerm = $8;
      $$->penalty = 0.0;
      $$->formula = $11;
      $$->active = TRUE;
      $$->filename = parseCurrentFile;
      $$->lineNo = parseLineNo;
      $$->cfunction = NULL;
      variables = NULL;
      $$->is_context_sensitive = is_context_sensitive;
      is_context_sensitive = FALSE;
    }
  | '{' VarList '}' ':' STR ':' Formula
    { 
      yyLastValidFound( "constraint" );
      $$ = (Constraint)memMalloc( sizeof(ConstraintStruct) );
      $$->vars = $2;
      $$->sig = (Signature)memMalloc( sizeof(SignatureStruct) );
      $$->sig->arity = listLength( $2 );
      $$->sig->conn = theConnexion;
      $$->id = $5;
      $$->penaltyTerm = NULL;
      $$->penalty = 0.0;
      $$->section = inputDefaultSection;
      $$->formula = $7;
      $$->active = TRUE;
      $$->filename = parseCurrentFile;
      $$->lineNo = parseLineNo;
      $$->cfunction = NULL;
      variables = NULL;
      $$->is_context_sensitive = is_context_sensitive;
      is_context_sensitive = FALSE;
    }
;

Direction:  
    ':'
    {
      $$ = AnyDir; 
    }
  | 
    '|'
    {
      $$ = Nil; 
    }
  |
    '!'
    {
      $$ = NonNil; 
    }
  |
    '\\'
    {
      $$ = Left; 
    }
  |
    '/'
    {
      $$ = Right; 
    }
;

Disjunction:
    Disjunction '|' Value
    {
      $$ = listAppendItem( $1, $3 );
    }
  | Value
    {
      $$ = listPrependItem( NULL, $1 );
    }
;

Property:
    STR '=' STR
    {
      $$ = (Property)memMalloc( sizeof(PropertyStruct) );
      $$->name = $1;
      $$->value = $3;
    }
;

Properties:
    Properties ',' Property
    {
      $$ = listAppendItem( $1, $3 );
    }
  | Property
    {
      $$ = listPrependItem( NULL, $1 );
    }
;

PropertyList:
    '[' Properties ']'
    {
      $$ = $2;
    }
   | 
    {
      $$ = NULL;
    }
;

Formula:
    STR Connexion STR 
    {
      List l;

      yyLastValidFound( "formula (Connexion)" );
      $$ = (Formula)memMalloc( sizeof(FormulaStruct) );
      $$->type = FTConnexion;
      $$->data.connexion.c = $2;

      /* seek first variable */
      for ( l = variables;
	    l != NULL && strcmp( $1, ((VarInfo)listItem(l))->varname ) != 0;
	    l = listNext(l) )
	/* empty body */
	;
      if ( l == NULL ) {
	cdgPrintf(CDG_ERROR,
		  "ERROR: %s:%d: false use of undeclared variable `%s'\n", 
		  parseCurrentFile, parseLineNo, $1);
	parseNoOfErrors++;
	$$->data.connexion.var1 = NULL;
      } else {
	$$->data.connexion.var1 = listItem(l);
      }
      cdgFreeString( $1 );
      /* seek second variable */
      for ( l = variables;
	    l != NULL && strcmp( $3, ((VarInfo)listItem(l))->varname ) != 0;
	    l = listNext(l) )
	/* empty body */
	;
      if ( l == NULL ) {
	cdgPrintf(CDG_ERROR,
		  "ERROR: %s:%d: false use of undeclared variable `%s'\n", 
		  parseCurrentFile, parseLineNo, $3);
	parseNoOfErrors++;
	$$->data.connexion.var2 = NULL;
      } else {
	$$->data.connexion.var2 = listItem(l);
      }
      cdgFreeString( $3 );
    }
   | STR Direction
    {
      List l;

      yyLastValidFound( "formula (Direction)" );
      $$ = (Formula)memMalloc( sizeof(FormulaStruct) );
      $$->type = FTDirection;
      $$->data.direction.d = $2;

      /* seek variable */
      for ( l = variables;
	    l != NULL && strcmp( $1, ((VarInfo)listItem(l))->varname ) != 0;
	    l = listNext(l) )
	/* empty body */
	;
      if ( l == NULL ) {
	cdgPrintf(CDG_ERROR,
		  "ERROR: %s:%d: false use of undeclared variable `%s'\n", 
		  parseCurrentFile, parseLineNo, $1);
	parseNoOfErrors++;
	$$->data.direction.var = NULL;
      } else {
	$$->data.direction.var = listItem(l);
      }
      cdgFreeString( $1 );
    }
  |  CTRUE
    {
      yyLastValidFound( "formula (true)" );
      $$ = (Formula)memMalloc( sizeof(FormulaStruct) );
      $$->type = FTConstant;
      $$->data.constant = TRUE;
    }
  | CFALSE
    {
      yyLastValidFound( "formula (false)" );
      $$ = (Formula)memMalloc( sizeof(FormulaStruct) );
      $$->type = FTConstant;
      $$->data.constant = FALSE;
    }
  | '(' Formula ')'
    {
      $$ = $2;
    }
  | '~' Formula
    {
      yyLastValidFound( "formula (negation)" );
      $$ = (Formula)memMalloc( sizeof(FormulaStruct) );
      $$->type = FTNegation;
      $$->data.formula = $2;
    }
  | STR '(' TermList ')'
    {
      CDGPredicate p;
      yyLastValidFound( "formula (predicate)" );
      $$ = (Formula)memMalloc( sizeof(FormulaStruct) );
      $$->type = FTPredicate;
      $$->data.predicate.functor = $1;
      $$->data.predicate.function = NULL;
      $$->data.predicate.args = $3;      

      p = predGetFunction($1);
      if (!p) 
	{
	  cdgPrintf(CDG_ERROR,
		    "ERROR: %s:%d: predicate `%s' unknown\n", 
		    parseCurrentFile, parseLineNo, $1 );
	  parseNoOfErrors++;
	} else {
          $$->data.predicate.function = p->function;
          if(p->is_context_sensitive) {
            is_context_sensitive = TRUE;
          }
        }
    }
  | Formula '&' Formula
    { 
      yyLastValidFound( "formula (conjunction)" );
      $$ = (Formula)memMalloc( sizeof(FormulaStruct) );
      $$->type = FTConjunction;
      $$->data.junction.arg1 = $1;      
      $$->data.junction.arg2 = $3;
    }
  | Formula '|' Formula
    { 
      yyLastValidFound( "formula (disjunction)" );
      $$ = (Formula)memMalloc( sizeof(FormulaStruct) );
      $$->type = FTDisjunction;
      $$->data.junction.arg1 = $1;      
      $$->data.junction.arg2 = $3;
    }
  | Formula IMPL Formula 
    { 
      yyLastValidFound( "formula (implication)" );
      $$ = (Formula)memMalloc( sizeof(FormulaStruct) );
      $$->type = FTImplication;
      $$->data.junction.arg1 = $1;      
      $$->data.junction.arg2 = $3;
    }
  | Formula BIIMPL Formula
    { 
      yyLastValidFound( "formula (equivalence)" );
      $$ = (Formula)memMalloc( sizeof(FormulaStruct) );
      $$->type = FTBiimplication;
      $$->data.junction.arg1 = $1;      
      $$->data.junction.arg2 = $3;
    }
  | Term RelationType Term
    {
      yyLastValidFound( "formula (relation)" );
      $$ = (Formula)memMalloc( sizeof(FormulaStruct) );
      $$->type = $2;
      $$->data.relation.op1 = $1;      
      $$->data.relation.op2 = $3;
    }
;

Stringlist:
  Stringlist STR
  {
    $$ = listAppendItem($1, $2);
  }  
  | STR
  {
    $$ = listPrependItem(NULL, $1);
  }
;

Hashentry:
  Stringlist MAP Value
  {
    $$ = listAppendItem($1, $3);
  }
;

Hashentrylist:
  Hashentrylist ',' Hashentry
    {
      $$ = listAppendItem( $1, $3 );
    }
  | Hashentry
    {
      $$ = listPrependItem( NULL, $1 );
    }
;

Hierarchy:
    STR IMPL Sort
    {
      hierarchyDepth = 0;
      yyLastValidFound( "hierarchy(Sort)" );
      $$ = inputSortTreeToHierarchy( $1, $3 );
      $$->filename = parseCurrentFile;
      $$->lineNo = parseLineNo;
    }
  | STR IMPL SubsumptionList
    {
      hierarchyDepth = 0;
      yyLastValidFound( "hierarchy(SubsumptionList)" );
      $$ = inputSubsumptionListToHierarchy( $1, $3 );
      $$->filename = parseCurrentFile;
      $$->lineNo = parseLineNo;
    }
;

InfoString:
  /* Infostring is a Conjunction of module contributions */
   InfoString '(' STR InfoModule ')'
   {
      $$ = $1;
      $$->data.list = listAppendItem($1->data.list, newValueAV($3, $4) );
   }
 | '(' STR InfoModule ')'
   {
      $$ = (Value)memMalloc(sizeof(ValueStruct));
      $$->type = VTConjunction;      

      $$->data.list = listPrependItem(NULL, newValueAV($2, $3));  
   }
;

InfoModule:
 InfoModule InfoItem 
{ /* is a Conjunction of Values */
    $$ = $1;
    if ($2->type == VTAVNode) {
       $$->data.list = listAppendItem($1->data.list, $2);
    } else {
       /* Pack it into an attribute value pair */
       sprintf(buffer, "%d", listLength($1->data.list) + 1);
       $$->data.list = listAppendItem($1->data.list, newValueAV( buffer, $2));
    }
 }
| InfoItem
 {
   /* Module info is a Conjunction of Values */
    $$ = (Value)memMalloc(sizeof(ValueStruct)); 
    $$->type = VTConjunction;
    if ($1->type==VTAVNode) {
       $$->data.list = listPrependItem(NULL, $1);
    } else {
       /* Pack it into attribute value couple */
       $$->data.list = listPrependItem(NULL, newValueAV("1", $1));
    }
 }
;

InfoItem:
 '(' STR InfoModule ')' 
 {   /* InfoItem is a attribute value couple  */
    $$ = newValueAV($2, $3);
 }
| STR 
{  /* or a string */
    $$ = (Value)memMalloc(sizeof(ValueStruct));
    $$->type = VTString;
    $$->data.string = $1;
 }
| NUM 
{ /* or a number */
   $$ = (Value)memMalloc(sizeof(ValueStruct));
   $$->type = VTNumber;
   $$->data.number = $1;
 }  
;

InputSeq: 
    /* empty */
    {
      $$ = parseResult;
      levelCounter = 0;
      lastValidFound = NULL;
      lastValidLineNo = -1;
    }
  | InputSeq LexiconItem ';'
    {
      yyLastValidFound( "lexical entry & semicolon" );
      inputInsertLI($1, $2);
      $$ = $1;
    }
  | InputSeq Level ';'
    {
      yyLastValidFound( "level declaration & semicolon" );
      $1->levels = listAppendItem( $1->levels, $2 );
      $$ = $1;
    }
  | InputSeq Constraint ';'
    {
      yyLastValidFound( "constraint & semicolon" );
      if(hashPut($1->constraints, $2->id, $2 )) {
        cdgPrintf(CDG_WARNING, "WARNING: constraint `%s' redefined\n", $2->id);
      }
      $$ = $1;
    }
  | InputSeq Lattice ';'
    {
      yyLastValidFound( "word graph & semicolon" );
      /*
	if(!trivialise($2)) {
	cdgPrintf(CDG_ERROR,
		  "ERROR: %s:%d: excessive timepoint found in word graph `%s'\n", 
		  parseCurrentFile, parseLineNo, $2->id);
	parseNoOfErrors++;
      }
      */
      if(hashPut($1->lattices, $2->id, $2)){
        cdgPrintf(CDG_WARNING, "WARNING: lattice `%s' redefined\n", $2->id);
      }
      $$ = $1;
    }
  | InputSeq AnnoEntry ';'
    {
      yyLastValidFound( "annotation & semicolon" );
      if(hashPut($1->annotations, $2->id, $2)) {
        cdgPrintf(CDG_WARNING, "WARNING: annotation `%s' redefined\n", $2->id);
      }
      $$ = $1;
    }
  | InputSeq Parameter ';'
    {
      yyLastValidFound( "parameter & semicolon" );
      $1->parameters = listAppendItem( $1->parameters, $2 );
      $$ = $1;
    }
  | InputSeq Hierarchy ';'
    {
      yyLastValidFound( "hierarchy & semicolon" );
      if(hashPut($1->hierarchies, $2->id, $2)) {
        cdgPrintf(CDG_WARNING, "WARNING: hierarchy `%s' redefined\n", $2->id);
      }
      $$ = $1;
    }
  | InputSeq STR MAP Hashentrylist ';'
    {
      yyLastValidFound( "hash table & semicolon" );
      if(hashPut($1->maps, $2, inputHashFromList($4))) {
        cdgPrintf(CDG_WARNING, "WARNING: map `%s' redefined\n", $2);
      }
      $$ = $1;
    }
  | InputSeq error ';'
    {
      yyerrok;
    }
;

LabelList:
    LabelList ',' STR
    {
      $$ = listAppendItem( $1, $3 );
    }
  | STR
    {
      $$ = listPrependItem( NULL, $1 );
    }
;

Lattice:
    STR ':' ArcList
    {
      yyLastValidFound( "word graph" );
      $$ = (Lattice)memMalloc( sizeof(LatticeStruct) );
      $$->id = $1;
      $$->arcs = $3;
      $$->filename = parseCurrentFile;
      $$->lineNo = parseLineNo;
      timepoint = 0;
      arcCounter = 0;
      inputScanLattice($$);
    }
 |  TBEGIN_LATTICE
    ArcListVM 
    TEND_LATTICE
    {
      yyLastValidFound( "VM word graph" );
      $$ = (Lattice)memMalloc( sizeof(LatticeStruct) );

       /* a VM Lattice has no name so
          the name is constructed of the file name 
          and file position */
      snprintf(buffer, BUFFER_LIMIT, "%s_%04d", 
               basename(parseCurrentFile), 
               parseLineNo - listLength($2) - 1);

      $$->id = strRegister(buffer);
      $$->arcs = $2;
      $$->filename = parseCurrentFile;
      $$->lineNo = parseLineNo;
      timepoint = 0;      
      arcCounter = 0;
      vmAdjustAcousticScore($$);
      inputScanLattice($$);
    }
  
/*
  | STR ':' ArcList
    {
      $$ = (Lattice)memMalloc( sizeof(LatticeStruct) );
      $$->id = $1;
      $$->annoId = NULL;
      $$->arcs = $3;
      $$->filename = parseCurrentFile;
      $$->lineNo = parseLineNo;
      timepoint = 0;
      arcCounter = 0;
      }
*/
;
Level:
    STR PropertyList '#' LabelList
    {
      yyLastValidFound( "level declaration" );
      $$ = (Level)memMalloc( sizeof(LevelStruct) );
      $$->id = $1;
      $$->properties = $2;
      $$->labels = $4;
      $$->no = levelCounter++;
      $$->showflag = TRUE;
      $$->useflag = TRUE;
      $$->filename = parseCurrentFile;
      $$->lineNo = parseLineNo;
    }
;

LexiconItem: 
    STR ISDEFINEDBY STR ':' Value
    {
      yyLastValidFound( "lexical entry" );
      yywarning("Named lexical entries are deprecated.");
      $$ = newLexiconItem($3, $5, parseCurrentFile, parseLineNo);
    }
    |
    STR ISDEFINEDBY '[' ']'
    {
      Value v = (Value)memMalloc( sizeof(ValueStruct) );
      yyLastValidFound( "lexical entry" );
      v->type = VTConjunction;
      v->data.list = NULL;

      $$ = newLexiconItem($1, v, parseCurrentFile, parseLineNo);
    }
    |
    STR ISDEFINEDBY '[' Conjunction ']'
    {
      Value v = (Value)memMalloc( sizeof(ValueStruct) );
      yyLastValidFound( "lexical entry" );
      v->type = VTConjunction;
      v->data.list = $4;

      $$ = newLexiconItem($1, v, parseCurrentFile, parseLineNo);
    }
    |
    STR MATCHES '[' Conjunction ']'
    {
      Value v = (Value)memMalloc( sizeof(ValueStruct) );
      yyLastValidFound( "template" );
      v->type = VTConjunction;
      v->data.list = $4;
      $$ = newLexiconItem(strRegister(""), v, parseCurrentFile, parseLineNo);
      if( regcomp(&$$->re, $1, REG_EXTENDED)) {
        yyerror("Bad regular expression.");
      }
    }
;

Parameter:
    '[' VarList ']' ':' NUM ':' Formula
    { 
      yyLastValidFound( "parameter" );
      $$ = (Constraint)memMalloc( sizeof(ConstraintStruct) );
      $$->vars = $2;
      $$->sig = NULL;
      $$->id = NULL;
      $$->section = NULL;
      $$->penalty = $5;
      $$->formula = $7;

      $$->filename = parseCurrentFile;
      $$->lineNo = parseLineNo;
      variables = NULL;
    }
;

Path:
    Path ':' STR
    {
      yyLastValidFound( "path" );
      $$ = listAppendItem( $1, $3 );
    }
  | STR
    {
      yyLastValidFound( "path" );
      $$ = listPrependItem( NULL, $1 );
    }
  | Path ':' NUM
    {
      yyLastValidFound( "path" );
      sprintf(buffer, "%d",(int)$3);
      $$ = listAppendItem( $1, strRegister( buffer ) );
    }
  | NUM
    {
      yyLastValidFound( "path" );
      sprintf(buffer, "%d", (int)$1);
      $$ = listPrependItem( NULL, strRegister( buffer ) );
    }
;

RelationType:
    '='   { $$ = FTEqual; }
  | '>'   { $$ = FTGreater; }
  | '<'   { $$ = FTLess; }
  | GTEQ  { $$ = FTGreaterEqual; }
  | LSEQ  { $$ = FTLessEqual; }
  | NEQ   { $$ = FTNotEqual; }
;

Sort:
    STR
    {
      yyLastValidFound( "sort" );
      $$ = (Sort)memMalloc( sizeof( SortStruct ) );
      $$->id = $1;
      $$->depth = hierarchyDepth;
      $$->subsorts = NULL;
    }
  | STR '(' { hierarchyDepth++; } SortList ')' 
    {
      /* this is tricky in that Hierarchydepth is messed with
	 during the expansion of the rule. Luckily the 
	 '(' disambiguates the context immediately. */
      hierarchyDepth--;
      yyLastValidFound( "sort" );
      $$ = (Sort)memMalloc( sizeof( SortStruct ) );
      $$->id = $1;
      $$->depth = hierarchyDepth;
      $$->subsorts = $4;      
    }
;

SortList:
    SortList ',' Sort
    {
      yyLastValidFound( "sort list" );
      $$ = listAppendItem( $1, $3 );
    }
  | Sort
    {
      yyLastValidFound( "sort list" );
      $$ = listPrependItem( NULL, $1 );
    }
;

Specification:
    STR '/' STR
    {
      $$ = newSpecification(STTag, $1, $3, 0);
    }
  | STR IMPL STR IMPL NUM
    {
      $$ = newSpecification(STDep, $1, $3, $5);
    }
;

SpecSeq:
    /* empty */
    {
      $$ = (List)NULL;
    }
  | SpecSeq Specification
    {
      $$ = listAppendItem( $1, $2 );
    }
;

Subsumption:
    STR IMPL TypeSeq
    {
      $$ = (Subsumption)memMalloc( sizeof(SubsumptionStruct) );
      $$->type = $1;
      $$->subsumesFlag = TRUE;
      $$->types = $3;
      /*
      cdgPrintf(CDG_ERROR, "STR(%s) IMPL TypeSeq(", $1 );
      for ( variables = $3; variables != NULL; variables = listNext(variables) ) {
	cdgPrintf( CDG_ERROR, "%s ", (char *)listItem(variables) );
      }
      cdgPrintf( CDG_ERROR, ") found\n" );
      */
    }
  | STR LPMI TypeSeq 
    {
      $$ = (Subsumption)memMalloc( sizeof(SubsumptionStruct) );
      $$->types = $3;
      $$->subsumesFlag = FALSE;
      $$->type = $1;
      /*
      cdgPrintf(CDG_ERROR, "TypeSeq(");
      for ( variables = $1; variables != NULL; variables = listNext(variables) ) {
	cdgPrintf( CDG_ERROR, "%s ", (char *)listItem(variables) );
      }
      cdgPrintf(CDG_ERROR, ") LPMI STR(%s) found\n", $3 );
      */
    }
;

SubsumptionList:
    SubsumptionList ',' Subsumption
    {
      $$ = listAppendItem( $1, $3 );
    }
  | Subsumption
    {
      $$ = listPrependItem( NULL, $1 );
    }
;

Term:
    STR '^' Path
    {
      List l;

      yyLastValidFound( "term (^)" );
      $$ = (Term)memMalloc( sizeof(TermStruct) );
      $$->type = TTTopPeek;
      $$->data.peek.path = $3;

      /* seek variable */
      for ( l = variables;
	    l != NULL && strcmp( $1, ((VarInfo)listItem(l))->varname ) != 0;
	    l = listNext(l) )
	/* empty body */
	;
      if ( l == NULL ) {
	cdgPrintf(CDG_ERROR,
		  "ERROR: %s:%d: false use of undeclared variable `%s'\n", 
		  parseCurrentFile, parseLineNo, $1);
	parseNoOfErrors++;
	$$->data.peek.varinfo = NULL;
      } else {
	$$->data.peek.varinfo = listItem(l);
      }
      cdgFreeString( $1 );
    }
  | STR '@' Path
    {
      List l;

      yyLastValidFound( "term (@)" );
      $$ = (Term)memMalloc( sizeof(TermStruct) );
      $$->type = TTBottomPeek;
      $$->data.peek.path = $3;

      /* seek variable */
      for ( l = variables;
	    l != NULL && strcmp( $1, ((VarInfo)listItem(l))->varname ) != 0;
	    l = listNext(l) )
	/* empty body */
	;
      if ( l == NULL ) {
	cdgPrintf(CDG_ERROR,
		  "ERROR: %s:%d: false use of undeclared variable `%s'\n", 
		  parseCurrentFile, parseLineNo, $1);
	parseNoOfErrors++;
	$$->data.peek.varinfo = NULL;
      } else {
	$$->data.peek.varinfo = listItem(l);
      }
      cdgFreeString( $1 );
    }
  | STR '.' LABEL
    {
      List l;

      yyLastValidFound( "term (.label)" );
      $$ = (Term)memMalloc( sizeof(TermStruct) );
      $$->type = TTLabel;

      /* seek variable */
      for ( l = variables;
	    l != NULL && strcmp( $1, ((VarInfo)listItem(l))->varname ) != 0;
	    l = listNext(l) )
	/* empty body */
	;
      if ( l == NULL ) {
	cdgPrintf(CDG_ERROR,
		  "ERROR: %s:%d: false use of undeclared variable `%s'\n", 
		  parseCurrentFile, parseLineNo, $1);
	parseNoOfErrors++;
	$$->data.label = NULL;
      } else {
	$$->data.label = listItem(l);
      }
      cdgFreeString( $1 );      
    }
  | STR '.' LEVEL
    {
      List l;

      yyLastValidFound( "term (.level)" );
      $$ = (Term)memMalloc( sizeof(TermStruct) );
      $$->type = TTLevel;

      /* seek variable */
      for ( l = variables;
	    l != NULL && strcmp( $1, ((VarInfo)listItem(l))->varname ) != 0;
	    l = listNext(l) )
	/* empty body */
	;
      if ( l == NULL ) {
	cdgPrintf(CDG_ERROR,
		  "ERROR: %s:%d: false use of undeclared variable `%s'\n", 
		  parseCurrentFile, parseLineNo, $1);
	parseNoOfErrors++;
	$$->data.level = NULL;
      } else {
	$$->data.level = listItem(l);
      }
      cdgFreeString( $1 );      
    }
  | Term '+' Term
    {
      yyLastValidFound( "term (+)" );
      $$ = (Term)memMalloc( sizeof(TermStruct) );
      $$->type = TTAdd;
      $$->data.operation.op1 = $1;
      $$->data.operation.op2 = $3;
    }
  | Term '-' Term
    {
      yyLastValidFound( "term (-)" );
      $$ = (Term)memMalloc( sizeof(TermStruct) );
      $$->type = TTSubtract;
      $$->data.operation.op1 = $1;
      $$->data.operation.op2 = $3;
    }
  | Term '*' Term
    {
      yyLastValidFound( "term (*)" );
      $$ = (Term)memMalloc( sizeof(TermStruct) );
      $$->type = TTMultiply;
      $$->data.operation.op1 = $1;
      $$->data.operation.op2 = $3;
    }
  | Term '/' Term
    {
      yyLastValidFound( "term (/)" );
      $$ = (Term)memMalloc( sizeof(TermStruct) );
      $$->type = TTDivide;
      $$->data.operation.op1 = $1;
      $$->data.operation.op2 = $3;
    }
  | STR '(' TermList ')' 
    {
      CDGFunction f;
      yyLastValidFound( "term (function)" );
      $$ = (Term)memMalloc( sizeof(TermStruct) );
      $$->type = TTFunction;
      $$->data.function.functor = $1;
      $$->data.function.function = NULL;
      $$->data.function.args = $3;

      f = funcGetFunction($1);
      if (!f)
	{
	  cdgPrintf(CDG_ERROR,
		    "ERROR: %s:%d: function `%s' unknown\n", 
		    parseCurrentFile, parseLineNo, $1 );
	  parseNoOfErrors++;
	} else {
          $$->data.function.function = (PointerFunction *)f->function;
          if(f->is_context_sensitive) {
            is_context_sensitive = TRUE;
          }
        }
    }
  | '[' Term ']'
    {
      $$ = $2;
    }
  | STR
    {
      yyLastValidFound( "term (string)" );
      $$ = (Term)memMalloc( sizeof(TermStruct) );
      $$->type = TTString;
      $$->data.string = $1;
    }
  | NUM
    {
      yyLastValidFound( "term (number)" );
      $$ = (Term)memMalloc( sizeof(TermStruct) );
      $$->type = TTNumber;
      $$->data.number = $1;
    }
;

TermList:
  TermList ',' Term
    {
      $$ = listAppendItem( $1, $3 );
    }
  | Term
    {
      $$ = listPrependItem( NULL, $1 );
    }
;

TypeSeq:
    STR
    {
      $$ = listPrependItem( NULL, $1 );
    }
  | TypeSeq STR
    {
      $$ = listAppendItem( $1, $2 );
    }
;

Value:
    NUM
    {
      $$ = (Value)memMalloc( sizeof(ValueStruct) );
      $$->type = VTNumber;
      $$->data.number = $1;
    }
  | STR
    {
      $$ = (Value)memMalloc( sizeof(ValueStruct) );
      $$->type = VTString;
      $$->data.string = $1;
    }
  | AttributeValue 
    { 
      $$ = (Value)memMalloc( sizeof(ValueStruct) ); 
      $$->type = VTAVNode; 
      $$->data.avnode = (struct AVNodeStruct *)$1; 
    } 
  | '<' ValueList '>'
    {
      List l;

      $$ = (Value)memMalloc( sizeof(ValueStruct) );
      $$->type = VTList;
      $$->data.list = $2;
      /*
       * Ok, ValueList are special purpose Values.
       * I just don't want to do the checks in `assessMatch'
       * because that would be runtime.
       */
      for ( l = $2; l != NULL; l = listNext(listNext(l)) ) {
	if ( listItem(l) == NULL || listNext(l) == NULL || listItem(listNext(l)) == NULL ) {
	  cdgPrintf(CDG_ERROR,
		    "ERROR: %s:%d: error in list of values, odd number of items\n",
		    parseCurrentFile, parseLineNo );	  
	  $$->data.list = NULL;
	  listForeachFree( $2, freeValue );
          parseNoOfErrors++;
	  break;
	}
	if ( ((Value)listItem(l))->type != VTString ) {
	  cdgPrintf(CDG_ERROR,
		    "ERROR: %s:%d: error in list of values, not a string\n",
		    parseCurrentFile, parseLineNo );	  
	  $$->data.list = NULL;
	  listForeachFree( $2, freeValue );
          parseNoOfErrors++;
	  break;
	}
	if ( ((Value)listItem(listNext(l)))->type != VTNumber ) {
	  cdgPrintf(CDG_ERROR,
		    "ERROR: %s:%d: error in list of values, not a number\n",
		    parseCurrentFile, parseLineNo );	  
	  $$->data.list = NULL;
	  listForeachFree( $2, freeValue );
          parseNoOfErrors++;
	  break;
	}
      }
    }
  | '[' Conjunction ']'
    {
      $$ = (Value)memMalloc( sizeof(ValueStruct) );
      $$->type = VTConjunction;
      $$->data.list = $2;
    }
  | '(' Disjunction ')'
    {
      $$ = (Value)memMalloc( sizeof(ValueStruct) );
      $$->type = VTDisjunction;
      $$->index = 0;
      $$->data.list = $2;
    }
  | '#' NUM '(' Disjunction ')'
    {
      $$ = (Value)memMalloc( sizeof(ValueStruct) );
      $$->type = VTDisjunction;
      $$->index = $2;
      $$->data.list = $4;
    }
;

ValueList:
    ValueList ',' Value
    {
      $$ = listAppendItem( $1, $3 );
    }
  | Value
    {
      $$ = listPrependItem( NULL, $1 );
    }
;

VarInfo:
    STR Direction STR
    {
      yyLastValidFound( "variable declaration" );
      $$ = (VarInfo)memMalloc( sizeof(VarInfoStruct) );
      $$->varname = $1;
      $$->levelname = $3;
      $$->level = NULL;
      $$->levelvalue = NULL;
      $$->dir = $2;
   }
   | STR Direction
    {
      yyLastValidFound( "variable declaration" );
      $$ = (VarInfo)memMalloc( sizeof(VarInfoStruct) );
      $$->varname = $1;
      $$->levelname = NULL;
      $$->level = NULL;
      $$->levelvalue = NULL;
      $$->dir = $2;
   }
   | STR
    {
      yyLastValidFound( "variable declaration" );
      $$ = (VarInfo)memMalloc( sizeof(VarInfoStruct) );
      $$->varname = $1;
      $$->levelname = NULL;
      $$->level = NULL;
      $$->levelvalue = NULL;
      $$->dir = AnyDir;
   }
;
      
VarList:
    VarInfo Connexion VarInfo
    {
      variables = listAppendItem(listAppendItem( NULL, $1 ), $3);
      theConnexion = $2;
      $$ = variables;
    }
  | VarInfo
    {
      variables = listPrependItem( NULL, $1 );
      theConnexion = AnyCon;
      $$ = variables;
    }
;

%%

/* ----------------------------------------------------------------------
 * extract lines from a file and print them
 */
void linex(String name, int from, int to)
{
#define LINEXMAXLINE 4096
  FILE *f=NULL;
  char buffer[LINEXMAXLINE];
  int lno=0;
  
  f=fopen(name, "r");
  if (!f)
    {
      cdgPrintf(CDG_WARNING, "WARNING: linex: can't open `%s'\n", name);
      return;
    }

  while (!feof(f))
    {
      if (!fgets(buffer, LINEXMAXLINE, f)) { break; }

      if (buffer[strlen(buffer)-1]=='\n') { lno++; }

      if (strlen(buffer)>=LINEXMAXLINE-1)
	{
	  buffer[LINEXMAXLINE-6]=' ';
	  buffer[LINEXMAXLINE-5]='.';
	  buffer[LINEXMAXLINE-4]='.';
	  buffer[LINEXMAXLINE-3]='.';
	  buffer[LINEXMAXLINE-2]='\n';
	  buffer[LINEXMAXLINE-1]='\0';
	}
      if (lno>=from) { cdgPrintf(CDG_DEFAULT, "%s", buffer); }
      if (lno>=to) { break; }
    }
  fclose(f);
}

/* ----------------------------------------------------------------------
 * print error message
 */
void yyerror( String s )
{
  int from, to;
  
  /* display the part of the file in error */
  if((hkVerbosity & CDG_ERROR) != 0) {
    
    from = lastValidLineNo;

    /* if the error occured at the beginning of a line,
       parseLineNo does not point to the actual error line yet.
       So we display one more line just to make sure the output
       will be useful at all. */
    to = parseLineNo+1;

    /* display at least ten lines of context for the error */
    if(from + 10 > to) {  from = to -10; }
    /* linex doesn't like indices < 1 */
    if(from < 1) { from = 1; }
    linex(parseCurrentFile, from, to);
  }

  if ( strcmp( s, "parse error" ) == 0 ) {
    if ( lastValidFound != NULL ) 
      cdgPrintf(CDG_ERROR,
	 "ERROR: %s:%d: parse error behind a %s in line %d\n",
    	 parseCurrentFile, parseLineNo, lastValidFound, lastValidLineNo );
    else 
      cdgPrintf(CDG_ERROR,
	 "ERROR: %s:%d: parse error\n", parseCurrentFile, parseLineNo );      
  } else {
    cdgPrintf(CDG_ERROR,
      "ERROR: %s:%d: %s\n", parseCurrentFile, parseLineNo, s );
  }
  parseNoOfErrors++;
}

/* ----------------------------------------------------------------------
 * print warning message
 */
void yywarning( String s )
{
  static String lastWarning = NULL;
  static int repetitions = 0;

  s = strRegister(s);
  
  /* conflate consecutive warnings, daemon style */
  if(s == lastWarning) {
    repetitions++;
  } else {
    if(repetitions) {
      cdgPrintf(CDG_WARNING, "Last warning repeated %d times.\n", repetitions);
      repetitions = 0;
    }
    cdgPrintf(CDG_WARNING,
              "WARNING: %s:%d: %s\n", parseCurrentFile, parseLineNo, s );
  }
  parseNoOfWarnings++;
  lastWarning = s;
}

/* ----------------------------------------------------------------------
 * remember last valid structure found
 */
void yyLastValidFound( String s )
{
  lastValidFound = s;
  lastValidLineNo = parseLineNo;
}

