/* Copyright (C) 1997-2003 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without 
 * modifications, as long as this notice is preserved.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * $Id: scache.h,v 1.16 2003/09/29 07:57:49 micha Exp $
 */

/* ---------------------------------------------------------------------------
 * @addtogroup Scache 
 * @{
 */

#ifndef SCACHE_H
#define SCACHE_H

/* ---------------------------------------------------------------------- */
#include "cdg.h"
#include "levelvalue.h"

/* -- TYPEDEFINITIONS ---------------------------------------------------- */

#ifndef SWIG

/* ---------------------------------------------------------------------------
 * This structure administers a score for each pair of LVs in a
 * constraint net.
 */
typedef struct {
  int hits;		     /**< holds the number of successful requests 
			       to the cache. */
  int count;		     /**< holds the total number of LVs registered 
			       in the cache */
  int size;		     /**< maximum used capacity */
  int capacity;		     /**< currently available capacity */
  Number *data;		     /**<  points to the underlying vector of 
			       Numbers */
} ScoreCacheStruct;
/** Pointer to ScoreCacheStruct */
typedef ScoreCacheStruct *ScoreCache;
#endif

/* -- VARIABLES --------------------------------------------------------- */

extern Boolean scUseCache;

/* -- FUNCTIONS --------------------------------------------------------- */

#ifndef SWIG
extern ScoreCache scNew(int size);
extern void scDelete(ScoreCache cache);
extern Number scGetScore(ScoreCache cache, LevelValue a, LevelValue b);
extern void scSetScore(ScoreCache cache, LevelValue a, LevelValue b, 
		       Number score);
extern void scInitialize();
#endif

/* ---------------------------------------------------------------------- */
#endif /* don't insert anything after this #endif */
/**@} */
