/*
 * File: set.h
 * Author: Horia F. Pop
 * Purpose: header file for managememt of cdg variables
 * Birth: March 21, 2001
 * 
 * $Id: set.h,v 1.8 2003/07/03 08:14:57 foth Exp $
 */

#ifndef SET_H
#define SET_H

/* -- INCLUDES ---------------------------------------------------------- */
#include <blah.h>
#include "variables.h"

/* -- MACROS ------------------------------------------------------------ */
#define SET_NONE    	VAR_NONE
#define SET_BOOL    	VAR_BOOL
#define SET_INT     	VAR_INT
#define SET_FLOAT   	VAR_FLOAT
#define SET_STRING  	VAR_STRING
#define SET_UNSLONG	VAR_UNSLONG
#define SET_BIT		VAR_BIT
#define SET_ENUM		VAR_ENUM

/* -- TYPE DEFINITIONS -------------------------------------------------- */
/*
 * Opaque type definition for iterator object used by
 * - setIteratorNew
 * - setIteratorNextName
 * - setIteratorFree
 */
struct SetIteratorStruct;
typedef struct SetIteratorStruct SetIteratorStruct, *SetIterator;

/* -- VARIABLES --------------------------------------------------------- */
/* -- FUNCTIONS --------------------------------------------------------- */

#ifndef SWIG
/* ----------------------------------------------------------------------
 * Makes the parameter a new valid iterator object for iterating
 * through all registered variables. The general use is as follows:
 *
 * String s;
 * SetIterator vi;
 *
 * SetIteratorNew(&vi);
 * while (s=SetIteratorNextName(&vi)) { do something with s }
 * SetIteratorFree(&vi);
 */
extern void setIteratorNew(SetIterator *);

/* ----------------------------------------------------------------------
 * Returns the name of the next variable. Cf. above.
 */
extern String setIteratorNextName(SetIterator *);

/* ----------------------------------------------------------------------
 * Finally deletes an setIterator object. Cf. above.
 */
extern void setIteratorFree(SetIterator *);

/* ----------------------------------------------------------------------
 * Registers a variable. The arguments are the name, type, address, and 
 * address of validation function and callback function
 *
 * o the validation function is used to confirm that a value is valid
 * o the callback function is called after the value has been set
 *
 * o funcValidate("name", "value", &variable)
 * o funcCallback("name", "value")
 *
 * o if type is SET_BIT, the next argument is the bitmask
 * 
 * o the next arguments are pairs of string and value
 * o the last argument is NULL
 */
extern Boolean setRegister(String name, int type, Pointer ptr, 
			   BooleanFunction *funcConvert,
			   BooleanFunction *funcValidate,
			   VoidFunction *funcCallback,
			   ...);
/* ----------------------------------------------------------------------
 * Sets an alias to an already registered variable
 */
extern Boolean setAlias(String alias, String name);

/* ----------------------------------------------------------------------
 * Sets the named variable with a value based on the second argument
 * Returns TRUE if the named variable does exist and if the value is valid
 * It is setValue's responsibility to set the value; the validating function
 * only validates it
 */
extern Boolean setValue(String name, String value);

/* ----------------------------------------------------------------------
 * Prints to file the name and the value of the requested variable
 * Returns TRUE if the named variable does exist
 */
extern Boolean setPrint(unsigned long mode, String name);
#endif

/* ----------------------------------------------------------------------
 * Returns a list of Strings holding the names of all variables
 */
extern List setReturnListOfNames();

/* ----------------------------------------------------------------------
 * Returns the type of the given variable, or SET_NONE otherwise
 */
extern int setReturnType(String name);

extern void setFinalize();

/* ---------------------------------------------------------------------- */
#endif /* don't insert anything after this #endif */
