/*
 * File: statistics.c
 * Author: Ingo Schroeder
 * Purpose: statistics evaluation
 * Birth: 14/4/97
 *
 * $Id: statistics.c,v 1.24 2003/02/18 13:04:08 micha Exp $
 */

/* ---------------------------------------------------------------------- */
#include <configure.h>

#include <stdio.h>
#include <blah.h>
#include "cdg.h"
#include "hook.h"
#include "input.h"
#include "lexemgraph.h"
#include "constraintnet.h"
#include "eval.h"
#include "functions.h"
#include "predicates.h"
#include "statistics.h"
#include "set.h"

/* ---------------------------------------------------------------------- */
/* -- TYPE DEFINITIONS -------------------------------------------------- */
/* -- FORWARD DECLARATIONS ---------------------------------------------- */
/* -- VARIABLES --------------------------------------------------------- */
Boolean statUseStatisticsFlag = FALSE;

/* ---------------------------------------------------------------------- */

/* ----------------------------------------------------------------------
 * computes 
 *
 *                                    L(l_i \| c_i. c_{m_i})
 * L(c_i, l_i, c_{m_i} \| \Omega ) = -------------------------
 *                                    \sum L( l \| c_i, c_k )
 */
Number statUnary(LevelValue lv, ConstraintNet net)
{
  List l, p;
  Number sum, para;
  LexemNode modifiee;
  String label;
  int i;
  LexemGraph lg = net->lexemgraph;

  /*
   * inefficient, complicated, but quick (and dirty)
   * Some parameters (or at least) the denominator could be cached.
   */

  for ( p = inputCurrentGrammar->parameters; p != NULL; p = listNext(p) ) {
    net->statUnary++;
    if ( evalUnaryConstraint( (Constraint)listItem(p), net, NULL, lv) )
      break;
  }

  /* no parameter found, so what shall we do? */
  if ( p == NULL ) {
    if ( hkVerbosity & CDG_INFO ) {
      cdgPrintf(CDG_INFO, "INFO: no parameter found for " );
      lvPrint(CDG_INFO, lv, NULL);
      cdgPrintf(CDG_INFO, "\n" );
    }
    return( 0.0 );
  }

  para = ((Constraint)listItem(p))->penalty;

  /* save level value */
  modifiee = lv->modifiee;
  label = lv->label;

  sum = 0.0;
  for ( l = lv->level->labels; l != NULL; l = listNext(l) ) {
    lv->label = (String)listItem(l);
    /* 
     * This is just an approximation. Not all lexemnodes from the lexem 
     * graph are suitable. Use of distance??
     */
    for ( i = 0; i < vectorSize( lg->nodes ); i++ ) {
      lv->modifiee = (LexemNode)vectorElementAt( lg->nodes, i );
      if ( lv->modifiee == lv->modifier
	   ||
	   !lgCompatibleNodes( lg, lv->modifier, lv->modifiee )
	   )
	continue;
      for ( p = inputCurrentGrammar->parameters; p != NULL; p = listNext(p) ) {
	net->statUnary++;
	if ( evalUnaryConstraint( (Constraint)listItem(p), net, NULL, lv ) )
	  break;
      }
      if ( p != NULL ) 
	sum += ((Constraint)listItem(p))->penalty;
    }

    /* NIL as modifiee */
    lv->modifiee = NULL;
    for ( p = inputCurrentGrammar->parameters; p != NULL; p = listNext(p) ) {
      net->statUnary++;
      if ( evalUnaryConstraint( (Constraint)listItem(p), net, NULL, lv ) )
	break;
    }
    if ( p != NULL ) 
      sum += ((Constraint)listItem(p))->penalty;
  }
  /* restore level value */
  lv->label = label;
  lv->modifiee = modifiee;

  if ( sum < para ) {
    fprintf(stderr, "ERROR: invalid fraction\n" );
    abort();
  }

  if ( hkVerbosity & CDG_INFO ) {
    cdgPrintf(CDG_INFO, "INFO: parameter %f for ", para / sum );
    lvPrint(CDG_INFO, lv, NULL);
    cdgPrintf(CDG_INFO, "\n" );
  }

  return( para / sum );
}

/* ----------------------------------------------------------------------
 *
 */
void statInitialize()
{
  setRegister("statistics", SET_BOOL, &statUseStatisticsFlag, NULL, NULL, NULL, NULL);
}

/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */


