/* Copyright (C) 1997-2003 The BLAH Team <blah@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without 
 * modifications, as long as this notice is preserved.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * All the functions of the array module are tested

   1.arrayNew(int i,...)
   2.arraySetElement(Array a, Pointer new, ...)
   3.arrayGetElement(Array a, ...)
   4.arrayGetDimension(Array a, int dim)
   5.arrayClone(Array a)
   6.arrayDelete(Array a)
   7.arraySetAllElements(Array a, Pointer new)
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <assert.h>
#include <blah.h>

int main(int argc, char **argv)
{
  int i, j, k, l;
  int x;
  Array a,b;

  blahInitialize();

/* -------------------------------------------
 * Test for arraynew , arrayGetDimension
 */
  a = arrayNew(3, 4, 5, 6, 0);
  assert(arrayGetDimension(a,0)==3);
  assert(arrayGetDimension(a,1)==4);
  assert(arrayGetDimension(a,2)==5);
  assert(arrayGetDimension(a,3)==6);

/* --------------------------------------------
 * Test for arraySetAllElements
 */

  arraySetAllElements(a,(Pointer)0);
  for (i = 0; i < 3; i++) {
    for (j = 0; j < 4; j++) {
      for (k = 0; k < 5; k++) {
	for (l = 0; l < 6; l++) {
	assert(arrayGetElement(a,i,j,k,l)==0);
	}
      }
    }
  }

/* --------------------------------------------
 * Test for arraySetElement
 */


  for (i = 0; i < 3; i++) {
    for (j = 0; j < 4; j++) {
      for (k = 0; k < 5; k++) {
	for (l = 0; l < 6; l++) {
	 arraySetElement(a, (Pointer) (i * j * k * l), i, j, k, l);
	 assert((int)arrayGetElement(a,i,j,k,l)==(i*j*k*l));
	}
      }
    }
  }
  
/* --------------------------------------------
 * Test for arrayGetElement
 */

   for (i = 0; i < 3; i++) {
    for (j = 0; j < 4; j++) {
      for (k = 0; k < 5; k++) {
	for (l = 0; l < 6; l++) {
	  x = (int)arrayGetElement(a, i, j, k, l);
          assert(x == i * j * k * l);
	 }
      }
    }
  }

/* --------------------------------------------
 * Test for arrayClone
 */

  b=arrayClone(a);
  for (i = 0; i < 3; i++) {
    for (j = 0; j < 4; j++) {
      for (k = 0; k < 5; k++) {
	for (l = 0; l < 6; l++) {
	assert((int)arrayGetElement(b,i,j,k,l)==(int)arrayGetElement(a,i,j,k,l));
	assert(arrayGetDimension(b,0)==arrayGetDimension(a,0));
        assert(arrayGetDimension(b,1)==arrayGetDimension(a,1));
	assert(arrayGetDimension(b,2)==arrayGetDimension(a,2));
	assert(arrayGetDimension(b,3)==arrayGetDimension(a,3));
      }
    }
  }
}

/* ------------------------------------------
 * Test for arrayDelete
 */
  arrayDelete(a);
  arrayDelete(b);

  blahFinalize();
  return 0;
}
