/* Copyright (C) 1997-2003 The BLAH Team <blah@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without 
 * modifications, as long as this notice is preserved.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 * The functions of the bitstring module that are tested are 
   1.bitNew(int size)
   2.bitClone(BitString bs)
   3.bitCopy(BitString dst, BitString src)
   4.bitFree(BitString bs)
   5.bitSize(BitString bs)
   6.bitPrint(BitString bs)
   7.resize(BitString bs, int size)
   8.bitSet(BitString bs, int no)
   9.bitClear(BitString bs, int no)
   10.bitSetAll(BitString bs)
   11.bitClearAll(BitString bs)
   12.bitAnd(BitString a, BitString b)
   13.bitOr(BitString a, BitString b)
   14.bitAndCheck(BitString a, BitString b)
   15.bitTest(BitString bs, int no)
   16.bitAreAllCleared(BitString bs)
   17.bitAreAllSet(BitString bs)
 */

#ifdef HAVE_CONFIG_H
  #include <config.h>
#endif

#include <stdio.h>
#include <blah.h>
#include <assert.h>

void BITPRINT(BitString b)
{
  int i;

  for (i = 0; i < bitSize(b); i++) {
    printf("%d", i % 10);
  }
  printf("\n");
  bitPrint(b);
  printf("\n\n");
}

int main(int argc, char **argv)
{
  BitString a, b;
  int i,j;

  blahInitialize();
  a = bitNew(33);
 
 /* ----------------------------------------
  * Testing for the bitSet,bitTest
  */

  bitSet(a, 20);  /* Setting bit 20 in a */
  assert(bitTest(a, 20)==1);

 /* ----------------------------------------
  * Test for bitClear
  */

  bitClear(a, 20);
  assert(bitTest(a, 20)==0);

  /* ----------------------------------------
   * Test for bitClone, bitSize
   */

  b = bitClone(a);
  assert(bitSize(a)==bitSize(b));

  for (j=1; j < bitSize(b); j++)
  assert(bitTest(a, j) == bitTest(b,j));

  /* ----------------------------------------
   * Test for bitSet for 'a'
   */

  for (i = 0; i < 20; i++)
   {
    bitSet(a, i);
    }
  for (j=1; j < 20; j++)
  assert(bitTest(a, j) == 1);

  /* ----------------------------------------
   * Test for bitSet for cloned 'b'
   */

  for (i = 21; i < 33; i++)
   {
    bitSet(b, i);
    }

  for (j = 21; j < 33; j++)
  assert(bitTest(b, j) == 1);


  /* ----------------------------------------
   * Test the bitAndCheck 
   */

  assert(bitTest(a,20) == 0);
  assert(bitTest(b,20) == 0);
  assert(bitAndCheck(a, b) == 0);

  /* ----------------------------------------
   * Test for bitOr
   */

  bitOr(a, b);

  for (i = 21; i < 33; i++)
  {
   assert(bitTest(a,i)== 1);
  }

  /* ----------------------------------------
   * Test for bitAnd
   */

  bitAnd(a,b);

  for (i = 1; i < 20; i++)
  {
   assert(bitTest(a,i)== 0);
  }


  /* ----------------------------------------
   * Test for bitSetAll
   */

  bitSetAll(a);

  for (j=1; j < bitSize(a); j++)
  assert(bitTest(a, j) == 1);

  /* ----------------------------------------
   * Test for bitClearAll
   */

  bitClearAll(b);

  for (j=1; j < bitSize(b); j++)
  assert(bitTest(b, j) == 0);

  assert(bitAndCheck(a, b) == 0);

  /* ----------------------------------------
   * Test for bitCopy
   */

  bitCopy(b, a);
  assert(bitAndCheck(a, b) == 1);


  /* ----------------------------------------
   * Test for bitAreAllCleared
   */
   for (j=1; j < bitSize(a); j++)
   assert(bitTest(a, j) == 1);

   assert(bitAreAllCleared(a) == 0);

  /* ----------------------------------------
   * Test for bitAreAllSet
   */

   for (j=1; j < bitSize(a); j++)
   assert(bitTest(a, j) == 1);

   assert(bitAreAllSet(a) == 1);

  /* resize(a,21); */

  /* ----------------------------------------
   * Test for bitFree
   */
  bitFree(a);

  blahFinalize();
  return 0;
}
