/* Copyright (C) 1997-2003 The BLAH Team <blah@nats.informatik.uni-hamburg.de>
 *
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without
 * modifications, as long as this notice is preserved.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Functions of the BitVector module tested
 *
 * bvNew
 * bvAddElement
 * bvElementAt
 * bvRemoveElementAt
 * bvInsertElementAt
 * bvSetElementAt
 * bvSetElements
 * bvSetAllElements
 * bvCapacity
 * bvSize
 * bvClone
 * bvCopy
 * bvDelete
 *
 */

#ifdef HAVE_CONFIG_H
  #include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <blah.h>
#include <assert.h>

int main(int argc, char **argv)
{
  BitVector bv, bvAfterClone;
  int i, ind;
  char ch;
  BitVector bvAfterCopy = bvNew(20);

  blahInitialize();

  /* ---------------------------------------- Test for bvNew,bvCapacity */

  bv = bvNew(15);
  assert(bvCapacity(bv) == 15);

  /* ---------------------------------------- Test for bvSetElements */

  bvSetElements(bv, 1, 10, 15);
  for (i = 10; i < 15; i++) {
    assert(bvElementAt(bv, i) == 1);
  }

  /* ---------------------------------------- Test for bvAddElement,bvSize */

  ind = bvAddElement(bv, 1);
  assert(ind == 15);
  assert(bvSize(bv) == 16);

  /* ---------------------------------------- Test for bvSetElementAt */

  ch = bvSetElementAt(bv, 1, 9);
  assert(bvElementAt(bv, 9) == 1);
  assert(ch == 0);

  /* ---------------------------------------- Test for bvInsertElementAt */

  ch = bvInsertElementAt(bv, 1, 19);
  assert(bvElementAt(bv, 19) == 1);
  assert(ch == 0);
  assert(bvSize(bv) == 20);

  /* ---------------------------------------- Test for bvRemoveElementAt */

  assert(bvSize(bv) == 20);
  assert(bvElementAt(bv, 18) == 0);

  ch = bvRemoveElementAt(bv, 18);

  assert(bvElementAt(bv, 18) == 1);
  assert(ch == 0);
  assert(bvSize(bv) == 19);

  /* ---------------------------------------- Test for bvClone */

  bvAfterClone = bvClone(bv);
  for (i = 0; i < 19; i++) {
    assert(bvElementAt(bv, i) == bvElementAt(bvAfterClone, i));
  }
  assert(bvSize(bv) == bvSize(bvAfterClone));
  assert(bvCapacity(bv) == bvCapacity(bvAfterClone));

  /* ---------------------------------------- Test for bvSetAllElements */

  assert(bvSize(bvAfterClone) == 19);
  for (i = 0; i < 9; i++) {
    assert(bvElementAt(bvAfterClone, i) == 0);
  }

  bvSetAllElements(bvAfterClone, 1);

  for (i = 0; i < 19; i++) {
    assert(bvElementAt(bvAfterClone, i) == 1);
  }

  /* -------------------------------------- Test for bvCopy */

  bvAfterCopy = bvCopy(bvAfterCopy, bv);
  for (i = 0; i < 19; i++) {
    assert(bvElementAt(bv, i) == bvElementAt(bvAfterCopy, i));
  }
  assert(bvSize(bv) == bvSize(bvAfterCopy));
  assert(bvCapacity(bv) == bvCapacity(bvAfterCopy));

  /* -------------------------------------- BitVectors are deleted */

  bvDelete(bv);
  bvDelete(bvAfterClone);
  bvDelete(bvAfterCopy);

  blahFinalize();
  return 0;
}
