/* Copyright (C) 1997-2003 The BLAH Team <blah@nats.informatik.uni-hamburg.de>
 *
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without
 * modifications, as long as this notice is preserved.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/*
 * The functions tested in the List module
 *
 * listAppendItem
 * listNext
 * listItem
 * listClone
 * listContains
 * listEqualsList
 * listAddUniqueItem
 * listDeleteLastItem
 * listAppendList
 * listLastItem
 * listSort
 * listInsertSorted
 * listReverse
 * listNth
 * listDeleteItem
 * vecorToList
 * listToVector
 * listIndex
 * listSetItem
 * listSetNext
 */


#ifdef HAVE_CONFIG_H
  #include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <blah.h>
#include <assert.h>

char *a[] = {
  "null", "eins", "zwei", "drei", "vier", "fuenf", "sechs", "sieben",
  "acht", "neun", "zehn", "elf", "zwoelf", "dreizehn", "vierzehn"
};

char *b[] = {
  "fuenfzehn","sechzehn","seibzehn","achtzehn","neunzehn"
 };

Boolean cmpString(String a, String b) {
  if (a == NULL)
    return (FALSE);
  return (b == NULL || strcmp(a, b) < 0);
}

int main(int argc, char **argv)
{
  List list = NULL, list2 = NULL, reverse;
  List listFromVector, listAfterClone, tail;
  int i = 1, j, l2;
  Vector v = vectorNew(15);

  blahInitialize();

  /* 
   * ----------------------------------------
   * Test for listAppendItem
   * Creates lists list,list2
   */
  for (i = 0; i < 15; i++) {
    list = listAppendItem(list, a[i]);
  }
  assert(listLength(list) == 15);
  for (i = 0; i < 15; i++)
    assert(listContains(list, a[i]));

  for (j = 0; j < 5; j++) {
    list2 = listAppendItem(list2, b[j]);
  }
  assert(listLength(list2) == 5);
  for (j = 0; j < 5; j++)
    assert(listContains(list2, b[j]));

  /* 
   * ----------------------------------------
   * Test for listNext,listItem
   */

  assert(strcmp(listItem(list), "null") == 0);
  tail = listNext(list);
  assert(listLength(tail) == 14);
  assert(listContains(tail, a[14]));

  /* 
   * ----------------------------------------
   * Test for listClone,listContains
   * ListAfterClone is clone of list2
   */

  listAfterClone = listClone(list2);
  assert(listLength(listAfterClone) == 5);
  for (i = 0; i < 5; i++)
    assert(listContains(listAfterClone, b[i]));

  /* 
   * ----------------------------------------
   * Test for listEqualsList
   * Compares the list2 and listAfterClone
   */
  
  assert(listEqualsList(list2, listAfterClone) == 0);

  /* 
   * ---------------------------------------
   * Test for listAddUniqueItem
   * Adds an item into listAfterClone
   */

  assert(listContains(listAfterClone, a[2]) == FALSE);
  listAddUniqueItem(listAfterClone, a[2]);
  assert(listContains(listAfterClone, a[2]) == TRUE);
  assert(listAddUniqueItem(listAfterClone, a[2]) == listAfterClone);

  /* 
   * ----------------------------------------
   * Test for listDeleteLastItem
   */

  l2 = listLength(listAfterClone);
  assert(strcmp(listLastItem(listAfterClone), "zwei") == 0);
  listDeleteLastItem(listAfterClone);
  assert(listLength(listAfterClone) == (l2 - 1));
  assert(strcmp(listLastItem(listAfterClone), "neunzehn") == 0);
  listFree(listAfterClone);

  /* 
   * -----------------------------------------
   * Test listAppendList,listLastItem
   * appending list to list 2.
   */

  listAppendList(list2, list);
  assert(listLength(list2) == 20);
  assert(strcmp(listLastItem(list2), "vierzehn") == 0);

  /* 
   * ----------------------------------------
   * Test listSort
   */
  {
    List sortedList = listSort(list, cmpString);

    list = sortedList;
    assert(listLength(list) == 15);
    assert(strcmp(listLastItem(list), "zwoelf") == 0);

  }

  listFree(list2);
  /* 
   * -----------------------------------------
   * Test listReverse
   */

  reverse = listReverse(list);
  assert(listLength(reverse) == 15);
  assert(strcmp(listLastItem(reverse), "acht") == 0);

  /* 
   * ----------------------------------------
   * ListInsertSorted,listNth
   */

  listInsertSorted(list, "fuenfzehn", cmpString);
  assert(listLength(list) == 16);
  assert(strcmp(listNth(list, 6), "fuenf") == 0);
  assert(strcmp(listNth(list, 7), "fuenfzehn") == 0);
  assert(strcmp(listNth(list, 8), "neun") == 0);

  /* 
   * -----------------------------------------
   * listDeleteItem
   */

  listDeleteItem(list, "fuenfzehn");
  assert(listLength(list) == 15);
  assert(strcmp(listNth(list, 6), "fuenf") == 0);
  assert(strcmp(listNth(list, 7), "neun") == 0);

  /* 
   * -----------------------------------------
   * listToVector
   */

  v = listToVector(list);
  assert(vectorSize(v) == 15);
  for (i = 0; i < vectorSize(v); i++)
    assert(vectorContains(v, a[i]));

  /* 
   * ----------------------------------------
   * vectorToList,listIndex
   */

  listFromVector = vectorToList(v);
  assert(listLength(listFromVector) == 15);
  for (i = 0; i < listLength(listFromVector); i++) {
    assert(listContains(listFromVector, a[i]));
    assert(listIndex(listFromVector, a[i]) == listIndex(list, a[i]));
  }

  /* 
   * ----------------------------------------
   * Test for listSetItem
   */
  assert(strcmp(listNth(list, 1), "acht") == 0);
  listSetItem(list, a[2]);
  assert(strcmp(listNth(list, 1), "acht") != 0);
  assert(strcmp(listNth(list, 1), "zwei") == 0);
  assert(strcmp(listNth(list, 1), a[2]) == 0);

  /* 
   * Test for listSetNext
   */

  assert(strcmp(listNth(list, 1), "zwei") == 0);
  listSetNext(list, reverse);
  assert(strcmp(listNth(list, 3), "zwei") == 0);
  assert(strcmp(listNth(list, 4), "zehn") == 0);

#if 0
  /* 
   * ----------------------------------------
   * PRINT a list
   */
  for (l = list; l; l = listNext(l)) {
    printf("%s ", listItem(l));
  }

  printf("\n");
#endif

  blahFinalize();
  return 0;
}

