# Copyright (C) 1997-2003 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## CdgMenu - menu component of the XCDG application.
##
## \author Michael Daum (see also AUTHORS and THANKS for more)
## $Id: menu.tcl,v 1.31 2003/09/22 13:35:13 micha Exp $
## ----------------------------------------------------------------------------
class CdgMenu {
   inherit ::itk::Widget


   # public
   public method init_data {}

   constructor {args} {}; ## \type TclList

   # private 
   private method _setVerbosity {}
   private method _setFlag {name}; ## \type TclString
   private method _toggleSheets {}
   private method _toggleShell {}
   private method _load_dir_action {}
   private method _load_action {}
   private method _reload_action {}
   private method _run_action {}
   private method _reset_action {}
   private method _prefs_action {}
   private method _about_action {}
   private method _help_action {}

   private variable _hintsFlag 1
   private variable _infosFlag 1
   private variable _warningsFlag 1
   private variable _evalFlag 1
   private variable _searchresultFlag 1
   private variable _profileFlag 0
   private variable _errorFlag 1
   private variable _debugFlag 0
   private variable _traceFlag 0
   private variable _sheetsFlag 1
   private variable _shellFlag 1
   private variable _agendaNormalizationFlag 0
   private variable _useCacheFlag 1
   private variable _searchModifiesNetFlag 1
   private variable _showDeletedFlag 0
   private variable _sloppySubsumesWarningsFlag 0
   private variable _statisticsFlag 0
   private variable _sortNodesFlag 0
   private variable _manyWindowsFlag 0
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body CdgMenu::constructor {args} {
   
   itk_component add lmenu {
      frame $itk_interior.lmenu
   } {}

   itk_component add rmenu {
      frame $itk_interior.rmenu
   } {}

   # file menu
   itk_component add filebutton {
      menubutton $itk_component(lmenu).filebutton \
		-text "File" \
		-underline 0 \
		-menu $itk_component(lmenu).filebutton.menu
   } {
      keep -background -cursor
      rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
   }

   itk_component add filemenu {
      menu $itk_component(filebutton).menu 
   } {
      keep -background -cursor
      rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
   }

   $itk_component(filemenu) add command \
		-label "Load ..." \
		-underline 0 \
		-command [code $this _load_action]
   $itk_component(filemenu) add command \
		-label "Load dir ..." \
		-underline 0 \
		-command [code $this _load_dir_action]
   $itk_component(filemenu) add command \
		-label "Reload" \
		-underline 0 \
		-command [code $this _reload_action]
   $itk_component(filemenu) add command \
		-label "Run ..." \
		-underline 0 \
		-command [code $this _run_action]
   $itk_component(filemenu) add separator
   $itk_component(filemenu) add command \
		-label "Preferences ..." \
		-underline 0 \
		-command [code $this _prefs_action]

   $itk_component(filemenu) add separator
   $itk_component(filemenu) add command \
		-label "Reset ..." \
		-underline 0 \
		-command [code $this _reset_action]
   $itk_component(filemenu) add command \
		-label "Quit" \
		-underline 0 \
		-command cmd::Quit

   # settings menu
   itk_component add settingsbutton {
      menubutton $itk_component(lmenu).settingsbutton \
		-text "Settings" \
		-underline 0 \
		-menu $itk_component(lmenu).settingsbutton.menu
   } {
      keep -background -cursor
      rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
   }

   itk_component add settingsmenu {
      menu $itk_component(settingsbutton).menu 
   } {
      keep -background -cursor
      rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
   }

   $itk_component(settingsmenu) add checkbutton \
		  -label "Cache Scores" \
		  -variable [scope _useCacheFlag] \
		  -command [code $this _setFlag useCache]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Search modifies net" \
		  -variable [scope _searchModifiesNetFlag] \
		  -command [code $this _setFlag searchModifiesNet]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Show deleted values" \
		  -variable [scope _showDeletedFlag] \
		  -command [code $this _setFlag showDeleted]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Sloppy subsume warnings" \
		  -variable [scope _sloppySubsumesWarningsFlag] \
		  -command [code $this _setFlag sloppySubsumesWarnings]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Compute statistics" \
		  -variable [scope _statisticsFlag] \
		  -command [code $this _setFlag statistics]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Sort nodes" \
		  -variable [scope _sortNodesFlag] \
		  -command [code $this _setFlag sortNodes]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Many parse windows" \
		  -variable [scope _manyWindowsFlag] \
		  -command [code $this _setFlag manyWindows]
   $itk_component(settingsmenu) add separator
   $itk_component(settingsmenu) add checkbutton \
		  -label "Hints" \
		  -variable "[scope _hintsFlag]" \
		  -command [code $this _setVerbosity]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Infos" \
		  -variable "[scope _infosFlag]"  \
		  -command [code $this _setVerbosity]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Warnings" \
		  -variable "[scope _warningsFlag]" \
		  -command [code $this _setVerbosity]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Debug" \
		  -variable "[scope _debugFlag]" \
		  -command [code $this _setVerbosity]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Error" \
		  -variable "[scope _errorFlag]" \
		  -command [code $this _setVerbosity]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Trace" \
		  -variable "[scope _traceFlag]" \
		  -command [code $this _setFlag trace]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Profile" \
		  -variable "[scope _profileFlag]" \
		  -command [code $this _setVerbosity]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Eval" \
		  -variable "[scope _evalFlag]" \
		  -command [code $this _setVerbosity]
   $itk_component(settingsmenu) add checkbutton \
		  -label "Searchresults" \
		  -variable "[scope _searchresultFlag]" \
		  -command [code $this _setVerbosity]

   # Windows
   itk_component add windowbutton {
      menubutton $itk_component(lmenu).windowbutton \
		-text "Window" \
		-underline 0 \
		-menu $itk_component(lmenu).windowbutton.menu
   } {
      keep -background -cursor
      rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
   }

   itk_component add windowmenu {
      menu $itk_component(windowbutton).menu 
   } {
      keep -background -cursor
      rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
   }

   $itk_component(windowmenu) add checkbutton \
		-label "Datasheets" \
		-variable "[scope _sheetsFlag]" \
		-command [code $this _toggleSheets]
   $itk_component(windowmenu) add checkbutton \
		-label "Shell" \
		-variable "[scope _shellFlag]" \
		-command [code $this _toggleShell]

   # help menu
   itk_component add helpbutton {
      menubutton $itk_component(rmenu).helpbutton \
		-text "Help" \
		-underline 0 \
		-menu $itk_component(rmenu).helpbutton.menu
   } {
      keep -background -cursor
      rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
   }

   itk_component add helpmenu {
      menu $itk_component(helpbutton).menu 
   } {
      keep -background -cursor
      rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
   }
	
   $itk_component(helpmenu) add command \
		-label "About" \
		-command [code $this _about_action]
   $itk_component(helpmenu) add command \
		-label "Help" \
		-command [code $this _help_action]

   # register the helpmessages
   .cdgmain help sethelpstr \
      $itk_component(filemenu)\ 0 \
      "Load a cdg-file." \
      $itk_component(filemenu)\ 1 \
      "Execute a cdg-command-file" \
      $itk_component(filemenu)\ 3 \
      "Change general preferences" \
      $itk_component(filemenu)\ 5 \
      "Reset the application."  \
      $itk_component(filemenu)\ 6 \
      "Terminate the application."  \
      $itk_component(settingsmenu)\ 0 \
      "Toggle the use of the score cache of binary evaluations" \
      $itk_component(settingsmenu)\ 1 \
      "Nodevalues of the net are adjusted after a netsearch." \
      $itk_component(settingsmenu)\ 2 \
      "Show or don`t show deleted nodevalues when displaying a net." \
      $itk_component(settingsmenu)\ 3 \
      "Suppress warnings due to sloppy grammar writing." \
      $itk_component(settingsmenu)\ 4 \
      "Compute statistics when applying net-algorithms." \
      $itk_component(settingsmenu)\ 5 \
      "Toggle sorting of constraint nodes after they've been built." \
      $itk_component(settingsmenu)\ 7 \
      "Toggle display of HINTS." \
      $itk_component(settingsmenu)\ 8 \
      "Toggle display of INFOS." \
      $itk_component(settingsmenu)\ 9 \
      "Toggle display of WARNINGS." \
      $itk_component(settingsmenu)\ 10 \
      "Toggle display of DEBUGS." \
      $itk_component(settingsmenu)\ 11 \
      "Toggle display ERRORS." \
      $itk_component(settingsmenu)\ 12 \
      "Toggle display of error-traces in the shell." \
      $itk_component(settingsmenu)\ 13 \
      "Toggle display of profiling information." \
      $itk_component(settingsmenu)\ 14 \
      "Toggle display of evaluation process." \
      $itk_component(settingsmenu)\ 15 \
      "Toggle display of results in the log-window after\
       applying a net-algorithm."\
      $itk_component(helpmenu)\ 1 \
      "Show general information on xcdg" \
      $itk_component(helpmenu)\ 2 \
      "Onlinehelp"

   pack $itk_component(lmenu) \
	$itk_component(rmenu) -side left -expand 1 -fill x
   pack $itk_component(filebutton) \
        $itk_component(settingsbutton) \
        $itk_component(windowbutton) -side left -padx 5 -pady 5
   pack $itk_component(helpbutton) -side right -padx 5 -pady 5

   eval itk_initialize $args
   
   init_data
}

## ----------------------------------------------------------------------------
## initialize the verbosity flags
## ----------------------------------------------------------------------------
body CdgMenu::init_data {} {
   global CDG_HINT CDG_INFO CDG_WARNING CDG_EVAL CDG_SEARCHRESULT
   global CDG_PROFILE CDG_ERROR CDG_DEBUG
   global cnSortNodesMethod 
   
   set verbosity [hkVerbosity_get]

   set _hintsFlag [expr ($verbosity & $CDG_HINT) > 0]
   set _infosFlag [expr ($verbosity & $CDG_INFO) > 0]
   set _warningsFlag [expr ($verbosity & $CDG_WARNING) > 0]
   set _evalFlag [expr ($verbosity & $CDG_EVAL) > 0]
   set _searchresultFlag [expr ($verbosity & $CDG_SEARCHRESULT) > 0]
   set _profileFlag [expr ($verbosity & $CDG_PROFILE) > 0]
   set _debugFlag [expr ($verbosity & $CDG_DEBUG) > 0]
   set _errorFlag [expr ($verbosity & $CDG_ERROR) > 0]
   set _traceFlag [.cdgmain shell cget -showTrace]
   set _sheetsFlag 1
   set _shellFlag 1
   set _useCacheFlag [scUseCache_get]
   set _searchModifiesNetFlag [nsSearchModifiesNetFlag_get]
   set _showDeletedFlag [cnShowDeletedFlag_get]
   set _sloppySubsumesWarningsFlag [evalSloppySubsumesWarnings_get]
   set _statisticsFlag [statUseStatisticsFlag_get]
   set _sortNodesFlag $cnSortNodesMethod
   set _manyWindowsFlag 1

   return $verbosity
}

## ----------------------------------------------------------------------------
## set the named flag according to our local variables
## ----------------------------------------------------------------------------
body CdgMenu::_setFlag {name} {

  switch $name {
    "agendaNormalization" {
      ::cmd::Set normalizeAgenda $_agendaNormalizationFlag
    }
    "useCache" {
      ::cmd::Set cache $_useCacheFlag
    }
    "searchModifiesNet" {
      ::cmd::Set searchmodifiesnet $_searchModifiesNetFlag
    }
    "showDeleted" {
      ::cmd::Set showdeleted $_showDeletedFlag
    }
    "sloppySubsumesWarnings" {
      ::cmd::Set subsumesWarnings $_sloppySubsumesWarningsFlag
    }
    "sortNodes" {
      ::cmd::Set sortnodes $_sortNodesFlag
    }
    "manyWindows" {
      .cdgmain parses setManyWindows $_manyWindowsFlag
    }
    "trace" {
      ::cmd::Set trace $_traceFlag
    }
    default {
      error "unknown flag $name"
    }
  }
}

## ----------------------------------------------------------------------------
## set the verbosity according to our local variables
## ----------------------------------------------------------------------------
body CdgMenu::_setVerbosity {} {
   global CDG_HINT CDG_INFO CDG_WARNING CDG_EVAL CDG_SEARCHRESULT
   global CDG_PROFILE CDG_ERROR CDG_DEBUG CDG_DEFAULT
   hkVerbosity_set [expr $_hintsFlag * $CDG_HINT + \
			  $_infosFlag * $CDG_INFO + \
			  $_warningsFlag * $CDG_WARNING + \
			  $_evalFlag * $CDG_EVAL + \
			  $_searchresultFlag * $CDG_SEARCHRESULT + \
			  $_profileFlag * $CDG_PROFILE + \
			  $_debugFlag * $CDG_DEBUG + \
			  $_errorFlag * $CDG_ERROR + \
			  $CDG_DEFAULT]
}

## ----------------------------------------------------------------------------
## delegate command to CdgMain
## ----------------------------------------------------------------------------
body CdgMenu::_load_action {} {
   .cdgmain files load
}

## ----------------------------------------------------------------------------
## open a directory selection dialog.
## ----------------------------------------------------------------------------
body CdgMenu::_load_dir_action {} {
   .cdgmain files load_dir
}

## ----------------------------------------------------------------------------
## delegate command to CdgMain
## ----------------------------------------------------------------------------
body CdgMenu::_reload_action {} {
   .cdgmain files reload
}

## ----------------------------------------------------------------------------
## reset application
## ----------------------------------------------------------------------------
body CdgMenu::_reset_action {} {
   ::cmd::Reset
}

## ----------------------------------------------------------------------------
## load a script and execute it
## ----------------------------------------------------------------------------
body CdgMenu::_run_action {} {
   set types {
      {"CDG scripts"		{.run .scr .cdg}}
      {"CDG files"		{.cdg m4}	}
      {"All files"		*		}
   }
   set filename [tk_getOpenFile -filetypes $types -parent .]
   if {$filename == ""} { return }

   .cdgmain shell safeSource $filename
}

## ----------------------------------------------------------------------------
## delegate command to CdgMain
## ----------------------------------------------------------------------------
body CdgMenu::_prefs_action {} {
   .cdgmain prefs activate
}

## ----------------------------------------------------------------------------
## startup the about-dialog
## ----------------------------------------------------------------------------
body CdgMenu::_about_action {} {
  .cdgmain confirm \
      "\[X\]CDG parser, Copyright (C) 1997-2003 The CDG Team

The CDG parser comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions; type `license' for details."

}

## ----------------------------------------------------------------------------
## startup the help
## ----------------------------------------------------------------------------
body CdgMenu::_help_action {} {
   .cdgmain confirm "Help is available under cdg/doc/ ."
}

## ----------------------------------------------------------------------------
## show/hide data-sheets
## ----------------------------------------------------------------------------
body CdgMenu::_toggleSheets {} {
   set pane [.cdgmain component pane]
   if {$_sheetsFlag == 0} {
      if {$_shellFlag == 0} {
	 set _sheetsFlag 1
	 return
      }
      $pane hide 0
   } else {
      $pane show 0
      $pane fraction 70 30
      update idletask
      .cdgmain shell see end
   }
}

## ----------------------------------------------------------------------------
## show/hide shell
## ----------------------------------------------------------------------------
body CdgMenu::_toggleShell {} {
   set pane [.cdgmain component pane]
   if {$_shellFlag == 0} {
      if {$_sheetsFlag == 0} {
	 set _shellFlag 1
	 return
      }
      $pane hide 1
   } else {
      $pane show 1
      $pane fraction 70 30
      update idletask
      .cdgmain shell see end
   }
}
