#!/bin/sh
#\
exec wish "$0" "$@" 

# ----------------------------------------------------------------------------
# Copyright (C) 1997-2003 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# ----------------------------------------------------------------------------

## ----------------------------------------------------------------------------
## \mainpage The XCDG Reference Manual
## \image html xcdg.gif
## \author Michael Daum, Kilian A. Foth, Dietmar Fnning
##
## $Id: xcdg.tcl,v 1.88 2003/12/04 08:43:14 foth Exp $
##
## \section Overview Overview
## This is the programmers reference manual of XCDG, that is a documentation
## of the implementation. For a detailed usage overview please consult the 
## Xcdg User Manual.
##
## \section Copyright Copyright
## Copyright (C) 1997-2003 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
##
## \verbatim
## XCDG is licensed under the GPL.
##  
## This application is free software; as a special exception the author gives
## unlimited permission to copy and/or distribute it, with or without 
## modifications, as long as this notice is preserved.
## 
## This program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
## implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
## \endverbatim
##
## ----------------------------------------------------------------------------

## \comment

if {![info exists env(XCDG_PATH)]} {
   set env(XCDG_PATH) {.}
}

if {![file exists $env(XCDG_PATH)]} {
   puts "ERROR: can't locate xcdg-directory, please set XCDG_PATH"
   exit
}

set env(XCDG_PATH) [glob $env(XCDG_PATH)]
lappend auto_path $env(XCDG_PATH)

# load some libraries:

# XML parsing for tcl
package require tdom

# Itcl -- object-oriented extensions to Tcl
package require Iwidgets 

# required so we can just write "class foo {...}"
namespace import -force itcl::*

# otherwise we would have to write "itcl::class foo {...}"

# a better table widget than the standard one
package require Tktable

# libcdg calls
load $env(CDG_BINDINGS)

# get compatibility hacks
source $env(XCDG_PATH)/compat.tcl

# display the startup-screen
wm withdraw .
set st [StartUp .startup -title "Xcdg - Splash"] 

# this kind of statement updates the startup-screen's progress bar
$st step 2

## ----------------------------------------------------------------------------
## called by the flush-hook
## ----------------------------------------------------------------------------
proc cdgFlush {} {
   global cdgFlushClock

   set now [clock clicks]
   set duration [expr abs($now - $cdgFlushClock)]
   if {$duration > 150000} {
      update 
      set cdgFlushClock $now
   }
}

## ----------------------------------------------------------------------------
## process the command line options given to xcdg etc.
## ----------------------------------------------------------------------------
proc init_xcdg {} {
   global argv 

   # standard values 
   set noinit 0
   set userFlag 0
   set execFiles ""

   cdgXCDG_set 1

   # parse parameters - set values
   foreach {flag value} $argv {
      switch -- $flag {
	 -i {
	    if {"$value" == "noinit" || "$value" == "none" || "$value" == "NULL"} {
	      set noinit 1
	    } else {
	      lappend execFiles $value
	    }
	 }
	 -grammarpath {
	    .cdgmain configure -grammarpath $value
	 }
	 -noinit {
	    set noinit 1
	 }
	 default {
	    puts stderr "unknown option `$flag'"
	    puts stderr "usage: xcdg \[-i <scriptfile\]* \n\
			\t\[-grammarpath <path-to-grammars>\] \n\
			\t\[-noinit\]"
	    exit 1
	 }
      }
   }


   # execute actions specified in .xcdgrc
   if {$noinit == 0 && "$execFiles" == ""} {
     if {[file exists ./.xcdgrc]} {
      .cdgmain shell safeSource ./.xcdgrc 
     } else {
       if {[file exists ~/.xcdgrc]} {
         .cdgmain shell safeSource ~/.xcdgrc 
       }
     }
   }

   # reset shell
   .cdgmain shell resetCmd

   # execute collected files
   foreach file $execFiles {
      if {$file == "stdin"} {
	 while {![eof stdin]} {
	    .cdgmain shell safeEval [read stdin]
	 }
      } else {
	 if {[file exists $file]} {
	    .cdgmain shell safeSource $file
	 } else {
	    ::cmd::Puts "ERROR: can't find file \"$file\""
	 }
      }
   }

}

# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
# initial steps

. configure -highlightthickness 0
tk_focusFollowsMouse
if {[catch { option readfile $env(XCDG_PATH)/Cdgrc userDefault } errMsg]} {
   puts "error in reading Cdgrc: "
   puts $errMsg
   exit 1
}



# set the hooks
setHookCmd flush "cdgFlush"
setHookCmd printf "::cmd::Puts -nonewline"
setHookCmd gets ".cdgmain shell fgets"
setHookCmd progress ".cdgmain help showstr"
setHookCmd partialresult ".cdgmain parses handlePartialResult"
setHookCmd "IC interaction" ".cdgmain parses handleICinteraction"
setHookCmd reset ".cdgmain parses reset"

set cdgFlushClock 0
commandEval "hook flush on"
commandEval "hook printf on"

# build the application
CdgMain .cdgmain
pack .cdgmain -fill both -expand 1
destroy $st
scan [wm maxsize .] "%d %d" maxx maxy
set maxx [expr $maxx - 100]
wm geometry . ${maxx}x${maxy}+100+0
wm minsize . 800 600
wm title . "Xcdg - Main"
wm deiconify .
update idletask

# source bugfixes: don't rely on autoload
source $env(XCDG_PATH)/bugfix.tcl

# print some ugly messages
set xcdg_version {$Id: xcdg.tcl,v 1.88 2003/12/04 08:43:14 foth Exp $}

::cmd::Puts "
INFO: \[X\]CDG parser v[lindex $xcdg_version 2], [lindex $xcdg_version 3]
      For more information see
          http://nats-wiki.informatik.uni-hamburg.de/twiki/bin/view/Papa
      The CDG Team can be contacted at
          cdg@nats.informatik.uni-hamburg.de
"

init_xcdg

## \endcomment
