# Copyright (C) 1997-2003 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## \comment
itk::usual ListDialog {
   keep -background -cursor -foreground 
}
## \endcomment

## ----------------------------------------------------------------------------
## ListDialog - a dialog embedding a list.
## This dialog is used to temporarily display a list of objects. In YadaRanking
## data files with errors are displayed after a load action.
##
## \author Michael Daum
##
## $Id: ListDialog.tcl,v 1.6 2003/02/27 13:02:26 micha Exp $
## ----------------------------------------------------------------------------
class ListDialog {
   inherit iwidgets::Dialog


   public method activate {}
   public method insert {component args}; ## \type TclString, TclList
   public method get {}

   constructor {args} {}; ## \type TclList

   private method _selectionAction {}
   private method _okAction {}
   private method _cancelAction {}

   ## title of the dialog
   public variable title ""
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body ListDialog::constructor {args} {

   # reduce buttons
   hide "Help"
   hide "Apply"

   set childsite [childsite]
   
   itk_component add label {
      label $childsite.label
   } {
      keep -cursor -foreground 
      rename -text -labeltext labelText Text
   }

   itk_component add list {
      iwidgets::scrolledlistbox $childsite.list \
		-vscrollmode static \
		-hscrollmode dynamic \
		-selectioncommand [code $this _selectionAction] \
		-dblclickcommand [code $this _okAction] \
		-selectmode single
   } {
      usual
   }
   
   pack $itk_component(label) \
	-side top -expand 0 -fill x -pady 5 -anchor nw
   pack $itk_component(list) \
	-side top -expand 1 -fill both

   buttonconfigure OK -command [code $this _okAction]
   buttonconfigure Cancel -command [code $this _cancelAction]

   eval itk_initialize $args


   [[component list] component listbox] configure -takefocus 0
}

## ----------------------------------------------------------------------------
## selection slot.
## When one of the listed items is selected then this action is fired.
## \todo not yet implemented
## ----------------------------------------------------------------------------
body ListDialog::_selectionAction {} {
}

## ----------------------------------------------------------------------------
## ok slot.
## This method is called when the OK button is pressed or a listed item
## is double-clicked. Afterwards the dialog is deactivated returning a value
## of 1 to the activating call.
## \see activate()
## ----------------------------------------------------------------------------
body ListDialog::_okAction {} {
   deactivate 1
}

## ----------------------------------------------------------------------------
## cancel slot.
## This method is deactivating the dialog returning a value of 0 to the call
## that opened the dialog.
## \see activate()
## ----------------------------------------------------------------------------
body ListDialog::_cancelAction {} {
   deactivate 0
}

## ----------------------------------------------------------------------------
## activate the dialog object.
## This method calls Shell:activate() after centering the dialog on the
## master of this widget.
## \returns 0 or 1 depending on the users selection.
## \see _okAction(), _cancelAction()
## ----------------------------------------------------------------------------
body ListDialog::activate {} {
   center $itk_option(-master)

   return [join [Shell::activate]]
}

## ----------------------------------------------------------------------------
## delegate insert to a component.
## This method is mainly used to insert an item to the list component of this
## widget.
## \param component the name of the component, i.e. "list"
## \param args the list of things to be inserted into the \a component
## ----------------------------------------------------------------------------
body ListDialog::insert {component args} {
  eval $itk_component($component) insert $args
}

## ----------------------------------------------------------------------------
## get the current selection.
## This method returns the selected items from the list component.
## \returns a list of list items..
## ----------------------------------------------------------------------------
body ListDialog::get {} {
  set result ""
  foreach index [$itk_component(list) curselection] {
    lappend result [$itk_component(list) get $index]
  }

  return $result
}

## ----------------------------------------------------------------------------
## -title
## ----------------------------------------------------------------------------
configbody ListDialog::title {
  wm title $itk_component(hull) $itk_option(-title)
}


