# Copyright (C) 1997-2003 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## YadaGenerals - stuff that didn't fit anywhere else til now.
## This class just offers a gui for some settings that we'd like to
## access but no special gui like the YadaExperiments or the YadaGrammars are
## done yet. That is there is no YadaConfigItems that we deal with here. Therefore
## The YadaGenerals don't inherit YadaConfigDocument.
## \ingroup YadaConfigDocument
##
## \author Michael Daum
##
## $Id: YadaGenerals.tcl,v 1.11 2003/02/27 13:02:27 micha Exp $
## ----------------------------------------------------------------------------
class YadaGenerals {
  inherit itk::Widget YadaPlainDocument


  # variables ----------------------------------------------------------------

  ## percentage indicating a significant difference in a quotient of two numbers.
  public variable significance  1.1

  ## tolerated time difference which isn't significant for a profiling distinction
  public variable timeTolerance 500

  ## the name of an editor to launch when we want to edit something
  public variable editor ""

  ## flag indicating how to compute weirdness in the YadaRunnableDocument s.
  public variable weirdness 1

  # methods ------------------------------------------------------------------
  public method init {}

  constructor {args} {}; ## \type TclList

  private method _commit {}
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body YadaGenerals::constructor {args} {

  itk_component add frame {
    frame $itk_interior.frame \
      -borderwidth 0
  }
  
  itk_component add significanceLabel {
    label $itk_component(frame).significanceLabel \
      -text "Significance:" \
      -anchor nw
  }

  itk_component add significance {
    entry $itk_component(frame).significance \
      -textvariable [scope significance] \
      -borderwidth 2 \
      -state normal \
      -width 20 \
  } {}

  itk_component add timeToleranceLabel {
    label $itk_component(frame).timeToleranceLabel \
      -text "Time Tolerance:" \
      -anchor nw
  }

  itk_component add timeTolerance {
    entry $itk_component(frame).timeTolerance \
      -textvariable [scope timeTolerance] \
      -borderwidth 2 \
      -state normal \
      -width 20 \
  } {}

  itk_component add editorLabel {
    label $itk_component(frame).editorLabel \
      -text "Editor:" \
      -anchor nw
  }

  itk_component add editor {
    entry $itk_component(frame).editor \
      -textvariable [scope editor] \
      -borderwidth 2 \
      -state normal \
      -width 20 \
  } {}

  itk_component add weirdnessLabel {
    label $itk_component(frame).weirdnessLabel \
      -text "Weirdness:" -anchor nw
  } {}

  itk_component add weirdness {
    checkbutton $itk_component(frame).strictWeirdness \
      -text "strict" \
      -variable [scope weirdness] \
      -command [code $this _commit]
  } {}


  # binding
  bind $itk_component(timeTolerance) <Return> [code $this _commit]
  bind $itk_component(significance) <Return> [code $this _commit]
  bind $itk_component(editor) <Return> [code $this _commit]

  # packing + gridding
  pack $itk_component(frame) -fill both -expand 1 -padx 10 -pady 10
  grid columnconfigure $itk_component(frame) 0 -pad 10
  grid columnconfigure $itk_component(frame) 1 -weight 1
  grid rowconfigure $itk_component(frame) 4 -weight 1

  grid $itk_component(editorLabel) -sticky nw -row 0 -column 0 -pady 10 
  grid $itk_component(editor) -sticky nw -row 0 -column 1 -pady 10 

  grid $itk_component(significanceLabel) -sticky ew -row 1 -column 0 -pady 10
  grid $itk_component(significance) -sticky nw -row 1 -column 1 -pady 10 

  grid $itk_component(timeToleranceLabel) -sticky ew -row 2 -column 0 -pady 10
  grid $itk_component(timeTolerance) -sticky nw -row 2 -column 1 -pady 10 

  grid $itk_component(weirdnessLabel) -sticky ew -row 3 -column 0 -pady 10
  grid $itk_component(weirdness) -sticky nw -row 3 -column 1 -pady 10 


  eval itk_initialize $args
}

## ----------------------------------------------------------------------------
## init
## ----------------------------------------------------------------------------
body YadaGenerals::init {} {
  if {$_isInitialized} {
    return
  }
  global env
  chain
  configure -editor $env(YADA_EDITOR)
}

## ----------------------------------------------------------------------------
## _commit
## ----------------------------------------------------------------------------
body YadaGenerals::_commit {} {
  configure \
    -significance $significance \
    -timeTolerance $timeTolerance \
    -editor $editor \
    -weirdness $weirdness
}
